@@ -16,7 +16,7 @@ import sys
 import csv
 from pathlib import Path
 import numpy as np
-
+from math import isclose
 import matplotlib.pyplot as plt
 import matplotlib.lines as lines
 
@@ -66,7 +66,7 @@ class MagnetField:
 #     return data
 
 class Notebook_LEDET:
-    def __init__(self, nameMagnet, typeMagnet = 'Cos-Theta', verbose = False, recalc_field=True, headerLines =1):
+    def __init__(self, nameMagnet, typeMagnet='Cos-Theta', verbose=False, headerLines=1, **kwargs):
         self.AvailableConductor = ['Type2', 'W1', 'PSU1sn', 'PSU2ti']
         self.AvailableOptions = ['cos_theta', 'solenoid']
         self.AvailableTransients = ['FPA']
@@ -92,12 +92,77 @@ class Notebook_LEDET:
         self.cart_labels = {'x': "x (m)", 'y': "y (m)"}
         self.rad_labels = {'x': "r (m)", 'y': "z (m)"}
 
+        force_tnt = True
+        force_l = True
         if self.typeMagnet == 'solenoid':
-            self.solenoid_data = read_yaml('magnet', self.nameMagnet)
-            self.out_dir = os.path.join(Path(os.getcwd()).parent, self.nameMagnet)
+            self.solenoid_data = read_yaml('circuit', self.nameMagnet)
+            self.solenoid_data['iStartQuench'] = int(self.solenoid_data['iStartQuench'])
+            self.solenoid_data['magnet_name'] = self.nameMagnet
+            for block_name in self.solenoid_data['blocks'].keys():
+                self.solenoid_data['blocks'][block_name]['wire_name'] = self.solenoid_data['blocks'][block_name]['wire']
+                self.solenoid_data['blocks'][block_name]['wire'] = read_yaml('conductor', self.solenoid_data['blocks'][block_name]['wire'])
+
+                winding_cell_rad_size = float(self.solenoid_data['blocks'][block_name]['wire']['strand']['wBare_inGroup']) + 2 * float(
+                    self.solenoid_data['blocks'][block_name]['wire']['strand']['wIns_inGroup']) + float(self.solenoid_data['blocks'][block_name]['r_PrePreg'])
+                strand_ins_ax_size = float(self.solenoid_data['blocks'][block_name]['wire']['strand']['hBare_inGroup']) + 2 * float(self.solenoid_data['blocks'][block_name]['wire']['strand']['hIns_inGroup'])
+                self.solenoid_data['blocks'][block_name]['wire']['strand']['wIns_inGroup'] = float(self.solenoid_data['blocks'][block_name]['wire']['strand']['wIns_inGroup']) + float(self.solenoid_data['blocks'][block_name]['r_PrePreg']) / 2
+
+                try:
+                    self.solenoid_data['blocks'][block_name]['A1'] = float(self.solenoid_data['blocks'][block_name]['A1'])
+                except:
+                    ref_bloc_name = self.solenoid_data['blocks'][block_name]['A1']
+                    self.solenoid_data['blocks'][block_name]['A1'] = float(
+                        self.solenoid_data['blocks'][ref_bloc_name]['A2'])  # use reference block name for A1
+                if force_tnt:
+                    self.n_layers = int(self.solenoid_data['blocks'][block_name]['nl'])
+                    self.n_turns_per_layer = int(self.solenoid_data['blocks'][block_name]['ntpl'])
+                    self.solenoid_data['blocks'][block_name]['A2'] = self.solenoid_data['blocks'][block_name]['A1'] + winding_cell_rad_size * self.n_layers  # correct A2 to allow for defined number of layers
+                    if force_l:
+                        strand_ins_ax_size = (self.solenoid_data['blocks'][block_name]['B2'] - self.solenoid_data['blocks'][block_name]['B1']) / self.n_turns_per_layer  # correct insulation thickness in axial direction to allow for loos/tight wind
+                        hIns_inGroup = (strand_ins_ax_size - self.solenoid_data['blocks'][block_name]['wire']['strand']['hBare_inGroup']) / 2
+                        if hIns_inGroup > 0:
+                            self.solenoid_data['blocks'][block_name]['wire']['strand']['hIns_inGroup'] = hIns_inGroup
+                        else:
+                            print(block_name)
+                            raise Exception(
+                                f"Calculated hIns_inGroup for {self.solenoid_data['blocks'][block_name]} is {hIns_inGroup} and <0. Please check number of layers in the solenoid or switch to force_l = False")
+                    else:
+                        self.solenoid_data['blocks'][block_name]['B2'] = self.solenoid_data['blocks'][block_name][
+                                                  'B1'] + strand_ins_ax_size * self.n_turns_per_layer  # adjust B2 to allow for larger wire - unrealistic as winding mandrel is usually fixed
+                else:
+                    self.solenoid_data['blocks'][block_name].n_layers = int(np.rint((self.solenoid_data['blocks'][block_name]['A2'] - self.solenoid_data['blocks'][block_name]['A1']) / winding_cell_rad_size))
+                    self.solenoid_data['blocks'][block_name].n_turns_per_layer = int(np.rint((self.solenoid_data['blocks'][block_name]['B2'] - self.solenoid_data['blocks'][block_name]['B1']) / strand_ins_ax_size))
+            self.Magnet.Options.flag_typeWindings = 1
+            self.sol_obj = Solenoid_magnet(self.solenoid_data)
+            if kwargs['model_no']:
+                self.out_dir = os.path.join(Path(os.getcwd()).parent, self.nameMagnet, str(kwargs['model_no']))
+            else:
+                self.out_dir = os.path.join(Path(os.getcwd()).parent, self.nameMagnet)
+            Ind_matrix_file = os.path.join(self.out_dir, f"{self.nameMagnet}_selfMutualInductanceMatrix.csv")
             if not os.path.exists(self.out_dir):
                 os.makedirs(self.out_dir)
-            self._setUp_Solenoid(recalc_field)
+            if 'recalc_field' in kwargs:
+                if kwargs['recalc_field']:
+                    self.sol_obj.save_B_map(self.out_dir)
+                    self.sol_obj.save_L_M(Ind_matrix_file)
+            else:
+                self.sol_obj.save_B_map(self.out_dir)
+                self.sol_obj.save_L_M(Ind_matrix_file)
+            self.Magnet.Inputs.T00 = self.solenoid_data['T']  # [K]
+            self.Magnet.Inputs.I00 = self.sol_obj.current[0]
+            self.Magnet.Inputs.M_m = np.loadtxt(Ind_matrix_file, delimiter=',', skiprows=1)
+            seq = np.arange(self.sol_obj.section.size)
+            sec_uni = list(np.int_(np.unique(self.sol_obj.section)))
+            M_M = np.zeros_like(np.arange(len(sec_uni) ** 2).reshape(len(sec_uni), len(sec_uni)), dtype=np.float64)
+            for sec_n in sec_uni:
+                for sec_m in sec_uni:
+                    idx_n = seq[np.isin(self.sol_obj.section, sec_n)]
+                    idx_m = seq[np.isin(self.sol_obj.section, sec_m)]
+                    chunk = self.Magnet.Inputs.M_m[np.ix_(idx_n, idx_m)]
+                    res = np.sum(chunk)
+                    M_M[sec_n - 1, sec_m - 1] = res
+            self.Magnet.Inputs.M_InductanceBlock_m = self.Magnet.Inputs.M_m
+            self.Magnet.Inputs.M_m = M_M
 
         # Start loading
         self.nStrands, self.polarities_inGroup, self.nHalfTurns, self.nTurns, self.nGroups, self.nCoilSections = 0, 0, 0, 0, 0, 0
@@ -107,11 +172,11 @@ class Notebook_LEDET:
         self.elPairs_GroupTogether = np.array([])
 
         if self.typeMagnet == 'solenoid':    # this can be only done after _load_ROXIE has been called.
-            self.load_ConductorData(self.Solenoid.wires, self.Solenoid.wire_groups)
-            max_distance = 0.5E-3
+            self.load_ConductorData(self.sol_obj.wires, self.sol_obj.wire_groups)
+            max_distance = np.max([self.Magnet.Inputs.wIns_inGroup+self.Magnet.Inputs.wBare_inGroup, self.Magnet.Inputs.hIns_inGroup+self.Magnet.Inputs.hBare_inGroup])*1.01    # take largest 101% of largest wire size
             self.set_ThermalConnections(max_distance)
             self.set_ElectricalOrder([0], [0])  # solenoid does not use any of the inputs, so it does not matter
-            self.set_SimulationType('2D')
+            self.set_SimulationType('2D+1D')
         # Further variables
         self.ConductorTypes = []
         self.ConductorGroups = np.array([])
@@ -120,46 +185,13 @@ class Notebook_LEDET:
         self.flag_Busbar = 0
         return
 
-    def _setUp_Solenoid(self, recalc_field):
-        self.Magnet.Options.flag_typeWindings = 1
-        self.Solenoid = Solenoid_magnet(self.nameMagnet)
-        self.Magnet.flagIron = self.solenoid_data['flagIron']
-        self.Magnet.flagSelfField = self.solenoid_data['flagSelfField']
-        field_map_file = os.path.join(self.out_dir, f"{self.nameMagnet}_All_NoIron_NoSelfField.map2d")
-        Ind_matrix_file = os.path.join(self.out_dir, f"{self.nameMagnet}_selfMutualInductanceMatrix.csv")
-        if recalc_field:
-            if self.Magnet.flagIron == 1 or self.Magnet.flagSelfField == 1:
-                raise Exception(
-                    'Can not calculate magnetic field using soleno and account for self field or iron effect')
-            else:
-                self.Solenoid.save_B_map(field_map_file)
-                self.Solenoid.save_L_M(Ind_matrix_file)
-        self.Magnet.Inputs.T00 = self.solenoid_data['T']  # [K]
-        if np.all(np.isclose(self.Solenoid.current, self.Solenoid.current[0])):
-            self.Magnet.Inputs.I00 = self.Solenoid.current[0]
-        else:
-            raise Exception("Issue with different current in each coil block, this might be okay, but never checked")
-        self.Magnet.Inputs.M_m = np.loadtxt(Ind_matrix_file, delimiter=',', skiprows=1)
-        seq = np.arange(self.Solenoid.section.size)
-        sec_uni = list(np.int_(np.unique(self.Solenoid.section)))
-        M_M = np.zeros_like(np.arange(len(sec_uni) ** 2).reshape(len(sec_uni), len(sec_uni)), dtype=np.float64)
-        for sec_n in sec_uni:
-            for sec_m in sec_uni:
-                idx_n = seq[np.isin(self.Solenoid.section, sec_n)]
-                idx_m = seq[np.isin(self.Solenoid.section, sec_m)]
-                chunk = self.Magnet.Inputs.M_m[np.ix_(idx_n, idx_m)]
-                res = np.sum(chunk)
-                M_M[sec_n - 1, sec_m - 1] = res
-        self.Magnet.Inputs.M_InductanceBlock_m = self.Magnet.Inputs.M_m
-        self.Magnet.Inputs.M_m = M_M
-
     def _load_ROXIE(self):
         # Select ROXIE .cadata file with conductor data
         currentDirectory = Path(os.path.split(os.getcwd())[0])
         fileNameCadata = os.path.join(currentDirectory, 'resources', 'roxie_parser', 'roxie.cadata')
         fileName = self.nameMagnet + '_All_WithIron_WithSelfField.map2d'
         fileNameData = self.nameMagnet + '_All_WithIron_WithSelfField.data'
-        if self.typeMagnet == 'solenoid' and self.solenoid_data['flagIron']==0 and self.solenoid_data['flagSelfField']==0:
+        if self.typeMagnet == 'solenoid':# and self.solenoid_data['flagIron']==0 and self.solenoid_data['flagSelfField']==0:
             fileName = os.path.join(self.out_dir, f"{self.nameMagnet}_All_NoIron_NoSelfField.map2d")
 
         print('O - Loading field maps from: ', fileName)
@@ -218,8 +250,10 @@ class Notebook_LEDET:
 
         self.strandToGroup = np.int_(strandToGroup)
         self.strandToHalfTurn = np.int_(strandToHalfTurn)
-        self.halfTurnToTurn = self.strandToHalfTurn
-        #self.halfTurnToTurn = np.tile(np.arange(1, self.nTurns + 1), 2) - commented out as it is shorter than other
+        if self.typeMagnet == 'solenoid':
+            self.halfTurnToTurn = self.strandToHalfTurn
+        elif self.typeMagnet == 'Cos-Theta':
+            self.halfTurnToTurn = np.tile(np.arange(1, self.nTurns + 1), 2) # multi-pole geometry assigns the first half of half-turns as go-lines, and the second half as return-lines
 
         # Average half-turn positions
         x_ave = []
@@ -281,7 +315,7 @@ class Notebook_LEDET:
             print('You are providing ', len(Conductor_L), '  conductor types, but only assign ',
                   np.max(np.array(Turns_L)), ' conductor. Please check!' )
             return
-
+        wire_area = []
         for i in range(len(Conductor_L)):
             Conductor = Conductor_L[i]
             Group_Mask = np.where(Turns_L==i+1)[0]
@@ -290,27 +324,31 @@ class Notebook_LEDET:
             if not Conductor in self.AvailableConductor:
                 print('Conductor type: ',Conductor, ' not understood. Available conductor: ', self.AvailableConductor)
                 return
-            conductor_data = read_yaml('conductor', Conductor)
-            cable_data = conductor_data['cable']
-            strand_data = conductor_data['strand']
+            if self.typeMagnet == 'solenoid':
+                #conductor_data = self.sol_obj.magnet_dict[i]['wire']
+                conductor_data = self.solenoid_data['blocks'][list(self.solenoid_data['blocks'].keys())[i]]['wire']
+            else:
+                conductor_data = read_yaml('conductor', Conductor)
+            #conductor_data['cable'] = conductor_data['cable']
+            #conductor_data['strand'] = conductor_data['strand']
             type = conductor_data['type']
-            SC = list(strand_data['SC_type'].keys())[0]
-            SC_data = strand_data['SC_type'][SC]
+            SC = list(conductor_data['strand']['SC_type'].keys())[0]
+            SC_data = conductor_data['strand']['SC_type'][SC]
 
             Values = np.array([0.0]*self.nGroups)
-            for key in cable_data.keys():
+            for key in conductor_data['cable'].keys():
                 if key in self.Magnet.Inputs.__annotations__:
                     try:
-                        Values[Group_Mask] = float(cable_data[key])
+                        Values[Group_Mask] = float(conductor_data['cable'][key])
                     except:
-                        Values[Group_Mask] = self.__translateDict(cable_data[key])
+                        Values[Group_Mask] = self.__translateDict(conductor_data['cable'][key])
                     self.Magnet.fillAttribute('Inputs', key, Values)
-            for key in strand_data.keys():
+            for key in conductor_data['strand'].keys():
                 if key in self.Magnet.Inputs.__annotations__:
                     try:
-                        Values[Group_Mask] = float(strand_data[key])
+                        Values[Group_Mask] = float(conductor_data['strand'][key])
                     except:
-                        Values[Group_Mask] = self.__translateDict(strand_data[key])
+                        Values[Group_Mask] = self.__translateDict(conductor_data['strand'][key])
                     self.Magnet.fillAttribute('Inputs', key, Values)
             for key in SC_data.keys():
                 if key in self.Magnet.Inputs.__annotations__:
@@ -323,62 +361,54 @@ class Notebook_LEDET:
             Values[Group_Mask] = self.__translateDict(SC)
             self.Magnet.fillAttribute('Inputs', 'SCtype_inGroup', Values)
 
+
             if type == 'rectangular':
-                wire_area = (self.Magnet.Inputs.wBare_inGroup[Group_Mask] * self.Magnet.Inputs.hBare_inGroup[Group_Mask]) \
-                            - ((4 - np.pi) * strand_data['scr_i'] ** 2)
-                eq_diam = np.sqrt(wire_area * 4 / np.pi)  # equivalent diameter with correcting for the corner radius
+                wire_area = conductor_data['strand']['wBare_inGroup'] * conductor_data['strand']['hBare_inGroup'] - ((4 - np.pi) * conductor_data['strand']['scr_b'] ** 2)
+                eq_diam = np.sqrt(wire_area * 4 / np.pi)
             else:
                 print('Conductor type: ', type, 'not supported.')
                 eq_diam = np.zeros(len(Group_Mask))  # TODO other conductor types, like cables, round monolith wires
-            Values = np.array([0.0] * self.nGroups)
+            #Values = np.array([0.0] * self.nGroups)
             Values[Group_Mask] = eq_diam
             self.Magnet.fillAttribute('Inputs', 'ds_inGroup', Values)
 
             try:
-                self.Magnet.Options.fScaling_Pex_AlongHeight = (2 * float(cable_data['insulationAroundCables'])) / (
-                            2 * float(cable_data['insulationAroundCables']) + float(cable_data['insulationBetweenLayers']))
+                self.Magnet.Options.fScaling_Pex_AlongHeight = (2 * float(conductor_data['cable']['insulationAroundCables'])) / (
+                            2 * float(conductor_data['cable']['insulationAroundCables']) + float(conductor_data['cable']['insulationBetweenLayers']))
             except:
                 self.Magnet.Options.fScaling_Pex_AlongHeight = 1
                 pass
             try:
-                if float(strand_data['T_Ref_RRR']) == 293:
+                if float(conductor_data['strand']['T_Ref_RRR']) == 293:
                     self.Magnet.Inputs.RRR_Cu_inGroup[Group_Mask]  = self.Magnet.Inputs.RRR_Cu_inGroup[Group_Mask] / 1.086
             except:
                 pass
 
-            if len(self.Magnet.Inputs.Jc_Nb3Sn0_inGroup)==0:
+            if len(self.Magnet.Inputs.Jc_Nb3Sn0_inGroup)!=self.nGroups:
                 self.Magnet.fillAttribute('Inputs', 'Jc_Nb3Sn0_inGroup',np.array([0]* self.nGroups))
-            if len(self.Magnet.Inputs.Bc2_Nb3Sn_inGroup)==0:
+            if len(self.Magnet.Inputs.Bc2_Nb3Sn_inGroup)!=self.nGroups:
                 self.Magnet.fillAttribute('Inputs', 'Bc2_Nb3Sn_inGroup',np.array([0]* self.nGroups))
-            if len(self.Magnet.Inputs.Bc2_NbTi_ht_inGroup)==0:
+            if len(self.Magnet.Inputs.Bc2_NbTi_ht_inGroup)!=self.nGroups:
                 self.Magnet.fillAttribute('Inputs', 'Bc2_NbTi_ht_inGroup',np.array([0]* self.nGroups))
-            if len(self.Magnet.Inputs.Tc0_Nb3Sn_inGroup)==0:
+            if len(self.Magnet.Inputs.Tc0_Nb3Sn_inGroup)!=self.nGroups:
                 self.Magnet.fillAttribute('Inputs', 'Tc0_Nb3Sn_inGroup',np.array([0]* self.nGroups))
-            if len(self.Magnet.Inputs.c2_Ic_NbTi_inGroup)==0:
+            if len(self.Magnet.Inputs.c2_Ic_NbTi_inGroup)!=self.nGroups:
                 self.Magnet.fillAttribute('Inputs', 'c2_Ic_NbTi_inGroup',np.array([0]* self.nGroups))
-            if len(self.Magnet.Inputs.c1_Ic_NbTi_inGroup)==0:
+            if len(self.Magnet.Inputs.c1_Ic_NbTi_inGroup)!=self.nGroups:
                 self.Magnet.fillAttribute('Inputs', 'c1_Ic_NbTi_inGroup',np.array([0]* self.nGroups))
-            if len(self.Magnet.Inputs.Tc0_NbTi_ht_inGroup)==0:
+            if len(self.Magnet.Inputs.Tc0_NbTi_ht_inGroup)!=self.nGroups:
                 self.Magnet.fillAttribute('Inputs', 'Tc0_NbTi_ht_inGroup',np.array([0]* self.nGroups))
 
+            if self.typeMagnet == 'solenoid':
+                if list(conductor_data['strand']['SC_type'].keys())[0]=="Nb-Ti":
+                    Jc_B_T = conductor_data['strand']['SC_type']['Nb-Ti']['c1_Ic_NbTi_inGroup']+conductor_data['strand']['SC_type']['Nb-Ti']['c2_Ic_NbTi_inGroup']*conductor_data['strand']['Ic_B']
+                    A_SC = conductor_data['strand']['Ic'] / Jc_B_T
+                    Values[Group_Mask] = A_SC/(wire_area*1e6)
+                    self.Magnet.fillAttribute('Inputs', 'f_SC_strand_inGroup', Values)
+
         if self.typeMagnet == 'solenoid':
-            self.Magnet.Inputs.ds_inGroup = []
-            self.Magnet.Inputs.nT = []
-            self.Magnet.Inputs.l_mag_inGroup = []
-            for block_name, block_dict in self.solenoid_data['blocks'].items():
-                conductor_data = read_yaml('conductor', block_dict['wire'])
-                block_dict['wire'] = conductor_data
-                sol = Solenoid(block_name, block_dict)
-                n_groups_in_block = np.shape(sol.Rin)[0]
-                wire_area = (conductor_data['strand']['wBare_inGroup'] * conductor_data['strand']['hBare_inGroup'])\
-                            - ((4 - np.pi) * conductor_data['strand']['scr_i'] ** 2)
-                eq_diam = np.sqrt(wire_area * 4 / np.pi)
-                self.Magnet.Inputs.ds_inGroup = self.Magnet.Inputs.ds_inGroup + ((np.ones(n_groups_in_block) * eq_diam).tolist()) # equivalent diameter
-                self.Magnet.Inputs.nT = self.Magnet.Inputs.nT + ((np.ones(n_groups_in_block, dtype=np.int32) * sol.n_turns_per_layer).tolist())  # Number of half-turns in each group
-                self.Magnet.Inputs.l_mag_inGroup = self.Magnet.Inputs.l_mag_inGroup + ((2 * np.pi * (sol.Rin + sol.Rout) / 2).tolist())  # Length of turns of each layer
-            self.Magnet.Inputs.nT = np.array(self.Magnet.Inputs.nT)
-            self.Magnet.Inputs.ds_inGroup = np.array(self.Magnet.Inputs.ds_inGroup)
-            self.Magnet.Inputs.l_mag_inGroup = np.array(self.Magnet.Inputs.l_mag_inGroup)
+            self.Magnet.Inputs.nT = self.sol_obj.ntpl
+            self.Magnet.Inputs.l_mag_inGroup = np.pi * (self.sol_obj.Rins + self.sol_obj.Routs)  # Length of turns of each layer
 
         self.ConductorTypes = Conductor_L
         self.ConductorGroups = Turns_L
@@ -390,8 +420,7 @@ class Notebook_LEDET:
             print('X - Calculation was successful.')
         elif self.typeMagnet == 'solenoid':
             self.nGroups = len(self.Magnet.Inputs.ds_inGroup)
-            self.Magnet.Inputs.GroupToCoilSection = self.Solenoid.section
-            #self.Magnet.Inputs.GroupToCoilSection = self.nGroups * [1]
+            self.Magnet.Inputs.GroupToCoilSection = self.sol_obj.section
             self.nCoilSections = np.int_(np.max(self.Magnet.Inputs.GroupToCoilSection))
             # Define to which inductive block each half-turn belongs
             self.Magnet.Inputs.HalfTurnToInductanceBlock = []
@@ -568,10 +597,17 @@ class Notebook_LEDET:
     def set_SimulationType(self, Scenario):
         print('O - Prepare simulation as:', Scenario)
         if Scenario == '2D+1D':
-            self.Magnet.Inputs.iStartQuench = np.linspace(1,self.nHalfTurns, self.nHalfTurns)
-            self.Magnet.Inputs.tStartQuench = np.ones((self.nHalfTurns,))*9999
-            self.Magnet.Inputs.lengthHotSpot_iStartQuench = np.ones((self.nHalfTurns,))*0.01
-            [_, self.MPZ, _, _, _, _] = self.Magnet.adjust_vQ(os.getcwd(), Return=1)
+            if self.typeMagnet == 'solenoid':
+                self.Magnet.Inputs.iStartQuench = np.array([int(self.solenoid_data['iStartQuench'])], dtype=np.int32)
+                file_path = os.path.join(self.out_dir, f"{self.nameMagnet}_All_NoIron_NoSelfField.map2d")
+                vQ, self.MPZ = self.Magnet.adjust_vQ(file_path)
+                self.Magnet.Inputs.vQ_iStartQuench = vQ[self.Magnet.Inputs.iStartQuench] * 2  # *2 to account for both directions of propagation
+                self.Magnet.Inputs.lengthHotSpot_iStartQuench = self.MPZ[self.Magnet.Inputs.iStartQuench]
+            else:
+                self.Magnet.Inputs.iStartQuench = np.linspace(1, self.nHalfTurns, self.nHalfTurns)
+                self.Magnet.Inputs.tStartQuench = np.ones((self.nHalfTurns,)) * 9999
+                self.Magnet.Inputs.lengthHotSpot_iStartQuench = np.ones((self.nHalfTurns,)) * 0.01
+                [_, self.MPZ, _, _, _, _] = self.Magnet.adjust_vQ(os.getcwd(), Return=1)
         elif Scenario == '2D':
             self.Magnet.Inputs.iStartQuench = np.array([1])
             self.Magnet.Inputs.tStartQuench = np.array([9999])
@@ -586,7 +622,7 @@ class Notebook_LEDET:
         self.SimulationType = Scenario
         print('X - Scenario prepared.')
 
-    def initiateQuench(self, Turn, TimeOfQuench, lengthHotspot = 0):
+    def initiateQuench(self, Turn, TimeOfQuench, lengthHotspot=0):
         print('O - Attempting to initiate quench.')
         if not isinstance(TimeOfQuench, list):
             TimeOfQuench = [TimeOfQuench]
@@ -812,44 +848,51 @@ class Notebook_LEDET:
             print('Transient type ', locals['Transient'], ' currently not supported or not understood.')
         print('X - Variables successfully loaded.')
 
-    def _solenoid_locals(self):
-        magnet_data = read_yaml('magnet', self.nameMagnet)
-        circuit_data = read_yaml('circuit', magnet_data['circuit'])
-        protection_data = read_yaml('protection', magnet_data['protection'])
-
-        R_EE_triggered = (protection_data['U_EE'] - self.Magnet.Inputs.I00 * (
-                circuit_data['R_circuit'] + circuit_data['R_crowbar']) - circuit_data['Ud_crowbar']) / self.Magnet.Inputs.I00  # Resistance of the energy-extraction system [Ohm]
-        if R_EE_triggered < 0:
+    def _solenoid_locals(self, t_PC, simul_time='file'):
+        if isclose(self.solenoid_data['U_EE'], 0.0):
+            tEE = 9999
             R_EE_triggered = 0
-        time_v_array = [item for sublist in [[rng['start'], rng['step'], rng['stop']] for rng in
-                        magnet_data['simul_time']] for item in sublist]
-        dict_block = {
+        else:
+            tEE = t_PC
+            R_EE_triggered = (self.solenoid_data['U_EE'] - self.Magnet.Inputs.I00 * (
+                    self.solenoid_data['R_circuit'] + self.solenoid_data['R_crowbar']) - self.solenoid_data['Ud_crowbar']) / self.Magnet.Inputs.I00  # Resistance of the energy-extraction system [Ohm]
+            if R_EE_triggered < 0:
+                R_EE_triggered = 0
+        if simul_time=='file':
+            time_v_array = [item for sublist in [[rng['start'], rng['step'], rng['stop']] for rng in
+                            self.solenoid_data['simul_time']] for item in sublist]
+        else:
+            time_v_array = simul_time
+        sim_start = time_v_array[0]
+        self.Magnet.Inputs.t_PC_LUT = [sim_start, t_PC, t_PC + 0.01]            # this overwrites the
+        solenoid_dict = {
+        't_PC': t_PC,
         'R_EE_triggered': R_EE_triggered,
         'alphasDEG': self.nHalfTurns * [0],
         'rotation_block': self.nHalfTurns * [0],
         'mirror_block': self.nHalfTurns * [0],
         'mirrorY_block': self.nHalfTurns * [0],
         #'initialQuenchTemp': 10,
-        'initialQuenchTemp': [1.],
+        'tEE': tEE,
+        'initialQuenchTemp': [10.],
         'tCLIQ': 99999,                  # Time when the CLIQ system is triggered [s]
-        'directionCurrentCLIQ': [1]* self.nCoilSections,    # Direction of the introduced current change for the chosen CLIQ configuration
+        'directionCurrentCLIQ': [1] * self.nCoilSections,    # Direction of the introduced current change for the chosen CLIQ configuration
         'nCLIQ': 0,                     # Number of CLIQ units
         'U0': 0,                        # CLIQ charging voltage [V]
         'C': 0,                         # Capacitance of the CLIQ capacitor bank [F]
         'Rcapa': 0,                      # Resistance of the CLIQ leads [Ohm]
         'fL_I': [0, 1000],
         'fL_L': [1, 1],
-        'tStartQuench': magnet_data['tStartQuench'],
-        'tQuench': magnet_data['tStartQuench'],
-        'iStartQuench': magnet_data['iStartQuench'],
-        'tStart': time_v_array[0],      # this is needed for power supply LUT
+        'tStartQuench': sim_start,#self.solenoid_data['tStartQuench'],
+        'tQuench': sim_start,#self.solenoid_data['tStartQuench'],
+        'tStart': sim_start,      # this is needed for power supply LUT
         'time_vector_params': time_v_array
         }
-        return {**circuit_data, **protection_data, **dict_block}
+        return {**solenoid_dict}
 
-    def writeLEDETFile(self, nameFileLEDET, locals):
+    def writeLEDETFile(self, nameFileLEDET, locals, **kwargs):
         if self.typeMagnet == 'solenoid':
-            locals = {**locals, **self._solenoid_locals()}
+            locals = {**locals, **self.solenoid_data, **self._solenoid_locals(**kwargs)}
             nameFileLEDET = os.path.join(self.out_dir, nameFileLEDET)
         self.storeVariables(locals)
         # Set flags depending on content:
@@ -878,20 +921,6 @@ class Notebook_LEDET:
         with open(batch_file_path, "w") as outfile:
             outfile.write("\n".join(str(line) for line in script_lines))
 
-    def run_LEDET_bat_parallel(self, model_no):
-        os.chdir(self.magnet_folder)
-        call_string = f"{self.magnet_name}_{int(model_no)}.bat"
-        subprocess.call([call_string])
-
-    def run_LEDET_bat_sequentially(self, model_no):
-        os.chdir(self.magnet_folder)
-        # LEDET_exe_path = os.path.join(LEDET_dir, LEDET_exe)
-        batch_file_path = os.path.join(os.getcwd(), f"{self.nameMagnet}_{model_no}.bat")
-        p = subprocess.Popen(batch_file_path, shell=False,
-                             stdout=subprocess.PIPE)
-        stdout, stderr = p.communicate()
-        print(p.returncode)
-
 
     ## Plotting part
     def plotter(self, data, titles, labels, types, texts, size=()):
@@ -1260,7 +1289,12 @@ class Notebook_LEDET:
 
         data = [{'x': self.MagnetGeometry.x_ave, 'y': self.MagnetGeometry.y_ave, 'z': ThConnections}]
         titles = ['Number of Thermal connections']
-        labels = [{'x': "x (m)", 'y': "y (m)", 'z': ""}] * len(data)
+        labels = [{'z': ""}] * len(data)
+        for label in labels:
+            if self.typeMagnet == 'solenoid':
+                label.update(self.rad_labels)
+            else:
+                label.update(self.cart_labels)
         types = ['scatter'] * len(data)
         texts = [self.text] * len(data)
         self.plotter(data, titles, labels, types, texts)
