@@ -104,24 +104,25 @@ class LEDETInputs:
     sim3D_fExToIns: float = 0.0
     sim3D_fExUD: float = 0.0
     sim3D_fExLR: float = 0.0
-    sim3D_min_ds_coarse: float = 0.0
-    sim3D_min_ds_fine: float = 0.0
-    sim3D_min_nodesPerStraightPart: float = 0.0
-    sim3D_min_nodesPerEndsPart: float = 0.0
-    sim3D_idxFinerMeshHalfTurn: float = 0.0
-    sim3D_Tpulse_sPosition: float = 0.0
-    sim3D_Tpulse_peakT: float = 0.0
-    sim3D_Tpulse_width: float = 0.0
-    sim3D_durationGIF: float = 0.0
-    sim3D_flag_saveFigures: float = 0.0
-    sim3D_flag_saveGIF: float = 0.0
-    sim3D_flag_VisualizeGeometry3D: float = 0.0
-    sim3D_flag_SaveGeometry3D: float = 0.0
+    sim3D_min_ds_coarse: float = 0.1
+    sim3D_min_ds_fine: float = 0.001
+    sim3D_min_nodesPerStraightPart: int = 4
+    sim3D_min_nodesPerEndsPart: int = 4
+    sim3D_idxFinerMeshHalfTurn: np.ndarray = np.array([])
+    sim3D_Tpulse_sPosition: float = 0.01
+    sim3D_Tpulse_peakT: float = 20
+    sim3D_Tpulse_width: float = 0.01
+    sim3D_durationGIF: float = 20
+    sim3D_flag_saveFigures: int = 1
+    sim3D_flag_saveGIF: int = 1
+    sim3D_flag_VisualizeGeometry3D: int = 1
+    sim3D_flag_SaveGeometry3D: int = 1
     M_m: np.ndarray = np.array([])
     fL_I: np.ndarray = np.array([])
     fL_L: np.ndarray = np.array([])
     HalfTurnToInductanceBlock: np.ndarray = np.array([])
     M_InductanceBlock_m: np.ndarray = np.array([])
+
 @dataclass
 class LEDETOptions:
     time_vector_params: np.ndarray = np.array([])
@@ -169,7 +170,9 @@ class LEDETOptions:
     flag_saveTxtFiles: float = 0.0
     flag_generateReport: float = 0.0
     flag_hotSpotTemperatureInEachGroup: float = 0.0
-    flag_3D: float = 0.0
+    flag_3D: int = 0
+    flag_adaptiveTimeStepping: int = 0
+
 @dataclass
 class LEDETPlots:
     suffixPlot: str = ''
@@ -548,6 +551,7 @@ class ParametersLEDET:
                 group = self.Variables
             else:
                 continue
+
             tt = type(self.getAttribute(group, attribute))
             if tt == np.ndarray and isinstance(locals[attribute], list):
                 self.setAttribute(group, attribute, np.array(locals[attribute]))
@@ -581,8 +585,7 @@ class ParametersLEDET:
         cpCu = density * cpCu_perMass
         return cpCu
 
-    def __rhoCu_nist(self, T, B, f_MR = 1):
-        RRR = np.repeat(self.Inputs.RRR_Cu_inGroup,self.Inputs.nT.astype(int))
+    def __rhoCu_nist(self, T, B, RRR, f_MR = 1):
         B = abs(B)
 
         idxLowB = np.where(B < 0.1)
@@ -849,8 +852,8 @@ class ParametersLEDET:
         Tc = np.zeros(B.shape)
         Tcs = np.zeros(B.shape)
         if len(idxNbTi)>0:
-            Tc[idxNbTi] = cable.Tc0_NbTi * (1 - B / cable.Bc20_NbTi) ** cable.alpha_NbTi
-            Tcs[idxNbTi] = (1 - I / (cable.c1_Ic_NbTi + cable.c2_Ic_NbTi * B)) * Tc
+            Tc[idxNbTi] = cable.Tc0_NbTi[idxNbTi] * (1 - B / cable.Bc20_NbTi[idxNbTi]) ** cable.alpha_NbTi
+            Tcs[idxNbTi] = (1 - I / (cable.c1_Ic_NbTi[idxNbTi] + cable.c2_Ic_NbTi[idxNbTi] * B[idxNbTi])) * Tc[idxNbTi]
         if len(idxNb3Sn) > 0:
             [Tc[idxNb3Sn], Tcs[idxNb3Sn]] = self.__Tc_Tcs_Nb3Sn_approx(I / cable.A_SC[idxNb3Sn], B[idxNb3Sn],
                                                                 cable.Jc_Nb3Sn0[idxNb3Sn], cable.Tc0_Nb3Sn[idxNb3Sn],
@@ -858,10 +861,10 @@ class ParametersLEDET:
 
         Ts = (Tcs + Tc) / 2
         cp_ST = np.zeros(B.shape)
-        cp_ST[idxCu_ST] = self.__cpCu_nist_mat(Ts)
+        cp_ST[idxCu_ST] = self.__cpCu_nist_mat(Ts[idxCu_ST])
         cp_SC = np.zeros(B.shape)
         if len(idxNbTi) > 0:
-            cp_SC[idxNbTi] = self.__cpNbTi_cudi_mat(Ts, B)
+            cp_SC[idxNbTi] = self.__cpNbTi_cudi_mat(Ts[idxNbTi], B[idxNbTi])
         if len(idxNb3Sn) > 0:
             cp_SC[idxNb3Sn] = self.__cpNb3Sn_alternative_mat(Ts[idxNb3Sn], B[idxNb3Sn], cable.Tc0_Nb3Sn[idxNb3Sn], cable.Bc20_Nb3Sn[idxNb3Sn])
         cp = cp_ST * f_ST_inStrand + cp_SC * f_SC_inStrand
@@ -870,9 +873,13 @@ class ParametersLEDET:
         vQ[idxInfQuenchVelocity]=1E6
 
         ### Calculate MPZ
-        rhoCu = self.__rhoCu_nist(Ts, B)
-        kCu = self.__kCu_WiedemannFranz(rhoCu, Ts)
-        l = np.sqrt((2 * kCu * (Tc - T_bath)) / (J_op ** 2 * rhoCu))
+        rhoCu = np.zeros(A_CableBare.shape)
+        kCu = np.zeros(A_CableBare.shape)
+        RRR = np.repeat(self.Inputs.RRR_Cu_inGroup, self.Inputs.nT.astype(int))
+        rhoCu[idxCu_ST] = self.__rhoCu_nist(Ts[idxCu_ST], B[idxCu_ST], RRR[idxCu_ST])
+        kCu[idxCu_ST] = self.__kCu_WiedemannFranz(rhoCu[idxCu_ST], Ts[idxCu_ST])
+        l = np.zeros(A_CableBare.shape)
+        l[idxCu_ST] = np.sqrt((2 * kCu[idxCu_ST] * (Tc[idxCu_ST] - T_bath)) / (J_op[idxCu_ST]** 2 * rhoCu[idxCu_ST]))
 
         # Calculate thermal conductivity of insulations
         idxKapton = np.where(self.Inputs.insulationType_inGroup == 2, 1, 0)
@@ -1036,11 +1043,33 @@ class ParametersLEDET:
         print('Minimum quench detection time would be {} ms [{} calculation]'.format(round(min(tQuenchDetection),3)*1000, Type))
         return min(tQuenchDetection)
 
-    def adjust_vQ(self, ROXIE_File, Transversaldelay  = False):
+    def getBField(self, ROXIE_File):
+        B = self._acquireBField(ROXIE_File)
+        strandCount = 0
+        GroupCount = 0
+        nStrands_inGroup = self.Inputs.nStrands_inGroup
+        ds_inGroup = self.Inputs.ds_inGroup
+        if any(nStrands_inGroup % 2 == 1) and any(nStrands_inGroup != 1):
+            for g in range(len(self.Inputs.nT)):
+                if (nStrands_inGroup[g] % 2 == 1) & (nStrands_inGroup[g] > 1):
+                    ds_inGroup[g] = ds_inGroup[g] * np.sqrt(nStrands_inGroup[g] / (nStrands_inGroup[g] - 1))
+                    nStrands_inGroup[g] = nStrands_inGroup[g] - 1
+
+        Bcopy = np.zeros(int(sum(self.Inputs.nT)))
+        for i in range(int(sum(self.Inputs.nT))):
+            Bcopy[i] = sum(B[int(strandCount):int(strandCount + nStrands_inGroup[GroupCount])]) / nStrands_inGroup[
+                int(GroupCount)]
+            TurnSum = sum(self.Inputs.nT[0:GroupCount + 1])
+            strandCount = strandCount + nStrands_inGroup[GroupCount]
+            if i > TurnSum: GroupCount = GroupCount + 1
+        return Bcopy
+
+    def adjust_vQ(self, ROXIE_File, Transversaldelay  = False, ManualB = '', CurrentsInCoilsections = []):
         cable = Cable()
         cable.A_CableInsulated = (self.Inputs.wBare_inGroup+2*self.Inputs.wIns_inGroup) \
                                * (self.Inputs.hBare_inGroup+2*self.Inputs.hIns_inGroup)
-        B = self._acquireBField(ROXIE_File)
+        if len(ManualB)==0: B = self._acquireBField(ROXIE_File)
+        else: B = ManualB
 
         if max(self.Inputs.nStrands_inGroup) > 1:
             strandCount = 0
@@ -1052,14 +1081,14 @@ class ParametersLEDET:
                     if (nStrands_inGroup[g] % 2 == 1) & (nStrands_inGroup[g] > 1):
                         ds_inGroup[g] = ds_inGroup[g] * np.sqrt(nStrands_inGroup[g] / (nStrands_inGroup[g] - 1))
                         nStrands_inGroup[g] = nStrands_inGroup[g] - 1
-
-            Bcopy = np.zeros(int(sum(self.Inputs.nT)))
-            for i in range(int(sum(self.Inputs.nT))):
-                Bcopy[i] = sum(B[int(strandCount):int(strandCount+nStrands_inGroup[GroupCount])])/nStrands_inGroup[int(GroupCount)]
-                TurnSum = sum(self.Inputs.nT[0:GroupCount+1])
-                strandCount = strandCount + nStrands_inGroup[GroupCount]
-                if i>TurnSum: GroupCount = GroupCount + 1
-            B = Bcopy
+            if len(ManualB)==0:
+                Bcopy = np.zeros(int(sum(self.Inputs.nT)))
+                for i in range(int(sum(self.Inputs.nT))):
+                    Bcopy[i] = sum(B[int(strandCount):int(strandCount+nStrands_inGroup[GroupCount])])/nStrands_inGroup[int(GroupCount)]
+                    TurnSum = sum(self.Inputs.nT[0:GroupCount+1])
+                    strandCount = strandCount + nStrands_inGroup[GroupCount]
+                    if i>TurnSum: GroupCount = GroupCount + 1
+                B = Bcopy
 
             cable.f_SC = self.Inputs.f_SC_strand_inGroup * \
                          (nStrands_inGroup* (np.pi/4)*(ds_inGroup**2)) / cable.A_CableInsulated
@@ -1071,7 +1100,6 @@ class ParametersLEDET:
             cable.f_ST = (1 - self.Inputs.f_SC_strand_inGroup) * \
                          (self.Inputs.wBare_inGroup * self.Inputs.hBare_inGroup) / cable.A_CableInsulated
 
-        I = self.Inputs.I00
         T_bath = self.Inputs.T00
         cable.A_SC =cable.A_CableInsulated * cable.f_SC
         cable.SCtype = self.Inputs.SCtype_inGroup
@@ -1086,15 +1114,68 @@ class ParametersLEDET:
         cable.Bc20_Nb3Sn = self.Inputs.Bc2_Nb3Sn_inGroup
         cable = self.__repeatCable(cable)
 
-        [vQ, l, HalfTurnToGroup, th_con_h, delta_t_h, th_con_w, delta_t_w] = self._quenchPropagationVelocity(I, B, T_bath, cable)
+        th_con_h = []
+        th_con_w = []
+        if len(CurrentsInCoilsections)>0:
+            if np.max(self.Inputs.GroupToCoilSection) != len(CurrentsInCoilsections):
+                print('You assigned ', len(CurrentsInCoilsections),' currents in the coilsections, but there are ',
+                      np.max(self.Inputs.GroupToCoilSection), ' Coil-sections. Abort!')
+                return
+
+            vQ_copy = np.linspace(0, len(cable.A_CableInsulated), len(cable.A_CableInsulated))
+            TurnToCoilSection = np.repeat(self.Inputs.GroupToCoilSection, self.Inputs.nT.astype(int))
+            for i in range(len(CurrentsInCoilsections)):
+                I = CurrentsInCoilsections[i]
+                B_copy = B/ self.Inputs.I00 * I
+                [vQ, l, HalfTurnToGroup, th_con_h, delta_t_h, th_con_w, delta_t_w] = \
+                    self._quenchPropagationVelocity(I, B_copy, T_bath, cable)
+
+                idx_cs = np.where(TurnToCoilSection == i+1)[0]
+                vQ_copy[idx_cs] = vQ[idx_cs]
+            vQ = vQ_copy
+        else:
+            I = self.Inputs.I00
+            [vQ, l, HalfTurnToGroup, th_con_h, delta_t_h, th_con_w, delta_t_w] = self._quenchPropagationVelocity(I, B, T_bath, cable)
+
         self.setAttribute(getattr(self, "Inputs"), "vQ_iStartQuench", vQ)
         #self.setAttribute(getattr(self, "Inputs"), "lengthHotSpot_iStartQuench", l)
         if Transversaldelay:
+            if len(CurrentsInCoilsections)>0:
+                print('Multiple currents in the coilsections are not supported for calculation of quench detection times. I use I_nom.')
             tQD = self.calculateQuenchDetectionTime(Transversaldelay, B, vQ, l, HalfTurnToGroup, th_con_h, delta_t_h,
                                               th_con_w, delta_t_w, uQuenchDetectionThreshold = 0.1)
             return [l, tQD]
         else:
-            return
+            return vQ, l
+
+    def adjust_vQ_QuenchHeater(self, th_con_h, th_con_w, NHeatingStations):
+        idx_turns_2x = np.array([])
+        activatedStrips = np.where(self.Inputs.tQH < 999)[0]+1
+        idx_turns_2x = np.append(idx_turns_2x, activatedStrips)
+
+        for i in activatedStrips:
+            idx_new_turns = np.where(self.Inputs.iQH_toHalfTurn_From == i)[0]
+            idx_turns_2x = np.append(idx_turns_2x, self.Inputs.iQH_toHalfTurn_To[idx_new_turns])
+        idx_turns_2x = idx_turns_2x.astype(int)
+        for j in range(5):
+            for i in idx_turns_2x:
+                if str(i) in th_con_h.keys():
+                    for k in th_con_h[str(i)]:
+                        if k not in idx_turns_2x:
+                            idx_turns_2x = np.append(idx_turns_2x, k)
+                if str(i) in th_con_w.keys():
+                    for k in th_con_w[str(i)]:
+                        if k not in idx_turns_2x:
+                            idx_turns_2x = np.append(idx_turns_2x, k)
+        idx_turns_2x = idx_turns_2x.astype(int)-1
+        self.Inputs.vQ_iStartQuench[idx_turns_2x] = self.Inputs.vQ_iStartQuench[idx_turns_2x] * NHeatingStations * 2
+        if len(self.Inputs.lengthHotSpot_iStartQuench) != len(self.Inputs.vQ_iStartQuench):
+            self.Inputs.lengthHotSpot_iStartQuench = np.array([0.01]*len(self.Inputs.vQ_iStartQuench))
+        if type(self.Inputs.lengthHotSpot_iStartQuench) != np.ndarray:
+            self.Inputs.lengthHotSpot_iStartQuench = np.array(self.Inputs.lengthHotSpot_iStartQuench)
+        #self.Inputs.lengthHotSpot_iStartQuench[idx_turns_2x] = np.array([0.01*NHeatingStations]*len(idx_turns_2x))
+        self.Inputs.lengthHotSpot_iStartQuench[idx_turns_2x] = np.array([self.Inputs.l_magnet*self.Inputs.f_QH[0]]*len(idx_turns_2x))
+        return
 
     def setHeliumFraction(self, PercentVoids):
         if np.max(self.Inputs.nStrands_inGroup)==1:
@@ -1128,6 +1209,9 @@ class ParametersLEDET:
         self.Inputs.overwrite_f_externalVoids_inGroup = extVoids
         self.Inputs.overwrite_f_internalVoids_inGroup = np.ones((nGroups,)).transpose()*(PercentVoids/100.0)
 
+        self.variablesInputs['overwrite_f_externalVoids_inGroup'] = 'Helium fraction in the external cable voids'
+        self.variablesInputs['overwrite_f_internalVoids_inGroup'] = 'Helium fraction in the internal cable voids'
+
     def preparePersistentCurrents(self, I_PC_LUT, dIdt, timeStep):
         # LUT controlling power supply, Current [A]. Two cycles of ramping from 0 to nominal current and back to zero
         if isinstance(I_PC_LUT,list):
@@ -1526,7 +1610,8 @@ class ParametersLEDET:
         if any(x > t4 for x in [t1, t2, t3, t5, t6, t7]):
             print("You're using a time, that is after the end of the simulation. Corrected Time-Vector.")
             self.Options.time_vector_params[-1] = np.max([t1, t2, t3, t5, t6, t7]) + 1
-            self.Inputs.t_PC_LUT[-1] = np.max([t1, t2, t3, t5, t6, t7]) + 1
+            self.Inputs.t_PC_LUT = np.append(self.Inputs.t_PC_LUT, np.max([t1, t2, t3, t5, t6, t7]) + 1)
+            self.Inputs.I_PC_LUT = np.append(self.Inputs.I_PC_LUT, self.Inputs.I_PC_LUT[-1])
         return 1
 
     def __checkPersistentCurrents(self):
@@ -1568,6 +1653,16 @@ class ParametersLEDET:
                 print('Unknown fit. Please check! Abort.')
                 return False
 
+    def __checkThermalConnections(self):
+        if len(self.Inputs.iContactAlongHeight_From) == 0 or len(self.Inputs.iContactAlongHeight_To) == 0:
+            self.Inputs.iContactAlongHeight_From = np.array([1])
+            self.Inputs.iContactAlongHeight_To = np.array([1])
+            print('No thermal connections in height directions set. I added at least 1.')
+        if len(self.Inputs.iContactAlongWidth_From) == 0 or len(self.Inputs.iContactAlongWidth_To) == 0:
+            self.Inputs.iContactAlongWidth_From = np.array([1])
+            self.Inputs.iContactAlongWidth_To = np.array([1])
+            print('No thermal connections in width directions set. I added at least 1.')
+
     def _consistencyCheckLEDET(self):
         '''
             **Consistency check of LEDET Inputs - Main function **
@@ -1585,11 +1680,11 @@ class ParametersLEDET:
         '''
         # Define groups that require the same size, number in each list contains the row-number of the attribute
         ## 0 Single - 1 CoilSections - 2 Groups - 3 Half-Turns - 4 doesn't matter - 5 iContactAlongWidth - 6 iContactAlongHeight - 7 vQlength
-        ## 8 Quench Heater, 9 QH_QuenchToFrom, 10 CLIQ,
+        ## 8 Quench Heater, 9 QH_QuenchToFrom, 10 CLIQ, 11 Persistent currents
         VarsSameInput = [['T00','l_magnet','I00','R_circuit','R_crowbar','Ud_crowbar','t_PC', 't_EE', 'R_EE_triggered', 'sim3D_uThreshold',
                           'sim3D_f_cooling_down','sim3D_f_cooling_up','sim3D_f_cooling_left','sim3D_f_cooling_right','sim3D_fExToIns',
                           'sim3D_fExUD','sim3D_fExLR','sim3D_min_ds_coarse','sim3D_min_ds_fine','sim3D_min_nodesPerStraightPart',
-                          'sim3D_min_nodesPerEndsPart','sim3D_idxFinerMeshHalfTurn','sim3D_Tpulse_sPosition','sim3D_Tpulse_peakT',
+                          'sim3D_min_nodesPerEndsPart','sim3D_Tpulse_sPosition','sim3D_Tpulse_peakT',
                           'sim3D_Tpulse_width','sim3D_durationGIF','sim3D_flag_saveFigures','sim3D_flag_saveGIF',
                           'sim3D_flag_VisualizeGeometry3D','sim3D_flag_SaveGeometry3D'],
                          ['M_m', 'directionCurrentCLIQ', 'tQuench', 'initialQuenchTemp'],
@@ -1599,7 +1694,7 @@ class ParametersLEDET:
                          'Lp_s_inGroup', 'R_c_inGroup', 'Tc0_NbTi_ht_inGroup', 'Bc2_NbTi_ht_inGroup', 'c1_Ic_NbTi_inGroup','c2_Ic_NbTi_inGroup',\
                          'Tc0_Nb3Sn_inGroup','Bc2_Nb3Sn_inGroup','Jc_Nb3Sn0_inGroup','el_order_half_turns'],
                         ['el_order_half_turns', 'alphasDEG', 'rotation_block', 'mirror_block', 'mirrorY_block','HalfTurnToInductanceBlock'],
-                        ['fL_I', 'fL_L', 'overwrite_f_internalVoids_inGroup', 'overwrite_f_externalVoids_inGroup','t_PC_LUT','I_PC_LUT'],
+                        ['fL_I', 'fL_L', 'overwrite_f_internalVoids_inGroup', 'overwrite_f_externalVoids_inGroup','t_PC_LUT','I_PC_LUT', 'sim3D_idxFinerMeshHalfTurn'],
                         ['iContactAlongWidth_From', 'iContactAlongWidth_To'],
                         ['iContactAlongHeight_From', 'iContactAlongHeight_To'],
                         ['iStartQuench', 'tStartQuench', 'lengthHotSpot_iStartQuench', 'vQ_iStartQuench'],
@@ -1746,6 +1841,10 @@ class ParametersLEDET:
         # Save the workbook
         std = workbook['Sheet']
         workbook.remove(std)
+
+        for s in range(len(workbook.sheetnames)):
+            if workbook.sheetnames[s] == 'Inputs': break
+        workbook.active = s
         workbook.save(nameFileLEDET)
 
         # Display time stamp and end run
@@ -1801,7 +1900,7 @@ class ParametersLEDET:
         # Correct section titles, if Helium option is present
         if "overwrite_f_externalVoids_inGroup" in variableGroup.__annotations__:
             ofiVg = self.getAttribute(variableGroup, "overwrite_f_externalVoids_inGroup")
-            if len(ofiVg) != 0:
+            if len(ofiVg) != 0 and "fitParameters_inGroup" in self.sectionTitles.keys():
                 self.sectionTitles["overwrite_f_externalVoids_inGroup"] = self.sectionTitles["fitParameters_inGroup"]
                 del self.sectionTitles["fitParameters_inGroup"]
 
