@@ -8,8 +8,9 @@ from nptdms import TdmsFile
 import nptdms
 from scipy import signal
 import csv
+import random
 from scipy.integrate import simps
-
+from datetime import datetime
 
 @dataclass
 class TDMSdata:
@@ -57,8 +58,10 @@ class SIMdata:
     TimeFrame: np.ndarray = np.array([])
     trigger_PC: int = 0
     XY_mag_ave: np.ndarray = np.array([])
+    B_turns: np.ndarray = np.array([])
     I_CoilSections: np.ndarray = np.array([])
     Uturn_half_turns: np.ndarray = np.array([])
+    U_PC: np.ndarray = np.array([])
     I_CLIQ: np.ndarray = np.array([])
     directionsCLIQ: np.ndarray = np.array([])
     U_CLIQ: np.ndarray = np.array([])
@@ -81,7 +84,7 @@ class WrappedVoltages:
 
 class QuenchPlanAnalysis():
     ## Domain: Helper and init function
-    def __init__(self, FileNameTDMS, FileNameSIMData, beforePC = 0.4, verbose: bool = False):
+    def __init__(self, FileNameTDMS, FileNameSIMData, beforePC = 0.04, verbose: bool = False):
         self.beforePC = beforePC
         self.Colors = self.__generateColors()
         self.verbose = verbose
@@ -101,12 +104,15 @@ class QuenchPlanAnalysis():
         # setting colors for plotting
         col = mcolors.TABLEAU_COLORS.keys()
         col2 = mcolors.BASE_COLORS.keys()
-        col3 = mcolors.CSS4_COLORS.keys()
+        col3 = [*mcolors.CSS4_COLORS.keys()]
+        random.shuffle(col3)
         colors = []
         for c in col:
             colors = colors + [c]
         for c in col3:
-            if c == 'white' or c == 'snow' or c == 'azure' or c == 'aliceblue': continue
+            if c == 'white' or c == 'snow' or c == 'azure' or c == 'aliceblue' or c=='cornsilk'\
+            or c=='lightyellow' or c=='beige' or c=='lavenderblush':
+                continue
             colors = colors + [c]
         for c in col2:
             if c == 'w': continue
@@ -277,7 +283,8 @@ class QuenchPlanAnalysis():
                 SimNumbers.append(index)
         return
 
-    def PrepareSimulation_FromMeasurements(self, RefFile, OutputDirectory, MagnetName = '', QHpairs = [[]], nQHseries = 0):
+    def PrepareSimulation_FromMeasurements(self, RefFile, OutputDirectory, MagnetName = '', CoilCsv = '', ROXIE = '',
+                                           QHpairs = [[]], nQHseries = 0):
         if not self.TDMSdata.FileName:
             print('Please first provide measured data. Abort.')
             return
@@ -290,6 +297,12 @@ class QuenchPlanAnalysis():
             MagnetName = RFile[idxRef:idxScore]
         if self.verbose: print("MagnetName = ",MagnetName)
 
+        if CoilCsv:
+            temp = ParametersLEDET()
+            temp.readLEDETExcel(RefFile)
+            self.SIMdata.el_connections = np.array([temp.Inputs.el_order_half_turns]).transpose()
+            self.ProvideTurnsToCoilStructure(CoilCsv = CoilCsv)
+
         a = ParametersLEDET()
         a.readLEDETExcel(RefFile)
 
@@ -305,6 +318,7 @@ class QuenchPlanAnalysis():
         # 2. Adjust Electrical parameters
         a.Inputs.R_circuit = self.TDMSdata.R_circuit
         a.Inputs.Ud_crowbar = self.TDMSdata.Ud_crow
+        a.Inputs.R_crowbar = self.TDMSdata.R_crow
 
         # 2.1 Adjust Quench Heater
         if np.any(self.TDMSdata.t_QH):
@@ -325,7 +339,10 @@ class QuenchPlanAnalysis():
                 for i in range(len(QHpairs)):
                     if len(QHpairs[i]) != nQHseries:
                         print("Provided Quench Heater pairs are not symmetric. I'm not sure how to handle this.")
-            U0_QH = np.max(self.TDMSdata.U_QH,axis=1)
+            U_QH = signal.medfilt(self.TDMSdata.U_QH, kernel_size=3)
+            U0_QH = np.max(U_QH,axis=1)
+            if self.verbose:
+                    print('Measured max. U_QH: ', U0_QH)
             rhoSS = a._rhoSS(a.Inputs.T00)
             R_cold_QH = (rhoSS /(a.Inputs.w_QH * a.Inputs.h_QH)) * a.Inputs.l_QH * a.Inputs.f_QH
 
@@ -346,7 +363,7 @@ class QuenchPlanAnalysis():
             a.Inputs.C = self.TDMSdata.C_CLIQ
             a.Inputs.U0 = np.amax(self.TDMSdata.U_CLIQ)
             for i in range(len(a.Inputs.directionCurrentCLIQ)):
-                idx_cs = np.where(a.Inputs.GroupToCoilSection == i+1)[0]
+                idx_cs = np.where(a.Inputs.GroupToCoilSection == i+1)[0][0]
                 ## TODO: That's not really right [-1*]
                 a.Inputs.directionCurrentCLIQ[i] = -1* a.Inputs.polarities_inGroup[idx_cs]
             ## TODO: R_CLIQ: See Notebook for attenuation calculation
@@ -358,7 +375,30 @@ class QuenchPlanAnalysis():
             a.Inputs.tEE = self.TDMSdata.t_EE
             a.Inputs.R_EE_triggered = self.TDMSdata.R_EE
 
-        a.writeFileLEDET(os.path.join(OutputDirectory, MagnetName + '_' + str(0) + '.xlsx'))
+        # 3. Adjust found quenches
+        if len(self.TDMSdata.Quenches)>0:
+            if not self.TDMSdata.Quenches[0, 1] != 0:
+                cont = 0
+                if ROXIE:
+                    self.SIMdata.B_turns = a.getBField(ROXIE)
+                elif self.SIMdata.B_turns.size ==0:
+                    print('Please either provide Magnetic field from .mat or provide ROXIE path. Quenches are not set.')
+                    cont = 1
+                if not cont:
+                    [l, _] = a.adjust_vQ('', Transversaldelay='Short', ManualB=self.SIMdata.B_turns)
+                    a.Inputs.lengthHotSpot_iStartQuench = l
+                    a.Inputs.iStartQuench = np.linspace(1, len(l), len(l))
+                    a.Inputs.tStartQuench = np.array([9999]*len(l))
+
+                    for i in range(len(self.TDMSdata.Quenches)):
+                        Turns = self.MergedVoltages[self.TDMSdata.Quenches[i,0]].astype(int)-1
+                        TurnMaxB = Turns[np.argmax(self.SIMdata.B_turns[Turns])]
+                        a.Inputs.tStartQuench[TurnMaxB -1] = self.TDMSdata.Quenches[i,1][0]
+                        if self.verbose: print('Quench in turn ',TurnMaxB,' at ',  np.round(self.TDMSdata.Quenches[i,1][0],3), ' s initiated.')
+
+        a.writeFileLEDET(os.path.join(OutputDirectory, MagnetName + '_' + str(0) + '.xlsx'), verbose = False)
+        currentDT = datetime.now()
+        print('File generated! Timestamp: ', currentDT)
 
     #############################################################################################################
     ## Domain: Functions used by both SIMdata/TMDSdata
@@ -445,7 +485,7 @@ class QuenchPlanAnalysis():
         for i in range(I.shape[1]):
             I_c = np.array([I[:,i]]).transpose()
             zeros = self.__zero_runs(I_c, Itol = dI_tol)
-            if not len(zeros) > 0 or abs(sum(I_c ) / len(I_c )) < 0.5 or zeros[0][1] == len(I):
+            if len(zeros) == 0 or np.max(I_c) < 10 or zeros[0][1] == len(I):
                 t_Start.append(np.nan)
                 if self.verbose: print('No signal found.')
                 continue
@@ -482,6 +522,7 @@ class QuenchPlanAnalysis():
                 idxTap = self.MergedVoltages[key]
                 legend.append(key)
                 ax.scatter(self.SIMdata.XY_mag_ave[0,idxTap.astype(int)-1], self.SIMdata.XY_mag_ave[1,idxTap.astype(int)-1], s = 10, c = self.Colors[count])
+                print(self.Colors[count])
                 count = count + 1
             ax.axis('equal')
             ax.grid('minor')
@@ -522,20 +563,21 @@ class QuenchPlanAnalysis():
                     if self.TDMSdata.VoltageVertices[i][6] ==  TapName or self.TDMSdata.VoltageVertices[i][7] == TapName:
                         realTapName = self.TDMSdata.VoltageVertices[i][8]
                 if not realTapName:
-                    print('Voltage Tap not found.')
+                    if self.verbose:  print('Voltage Tap not found.')
                     return
                 try:
                     MVidx = list(self.MergedVoltages.keys()).index(realTapName)
                 except:
                     try:
                         _ = self.TDMSdata.groupMF[realTapName]
-                        print('Skip Voltage-tap ', realTapName, ' as no turns are bound to it [E1: MagnetEnds]')
+                        if self.verbose: print('Skip Voltage-tap ', realTapName, ' as no turns are bound to it [E1: MagnetEnds]')
                         continue
                     except:
-                        print('Voltage tap ', realTapName, ' was not found in TDMS data. [E2: TDMS]')
+                        if self.verbose:  print('Voltage tap ', realTapName, ' was not found in TDMS data. [E2: TDMS]')
                         continue
                 newV = WrappedVoltages()
-                newV.SIM = np.sum(self.SIMdata.Uturn_half_turns[self.MergedVoltages[realTapName].astype(int)-1], axis=0)*self.MergedVoltagesPolarities[realTapName]
+                if self.SIMdata.FileName:
+                    newV.SIM = np.sum(self.SIMdata.Uturn_half_turns[self.MergedVoltages[realTapName].astype(int)-1], axis=0)*self.MergedVoltagesPolarities[realTapName]
                 newV.TDMS = self.TDMSdata.groupMF[realTapName].data[self.TDMSdata.trigger_shoot:]
                 # Correct for different length in Utaps and other channels
                 if len(newV.TDMS)<len(self.TDMSdata.TimeFrame_MF):
@@ -553,7 +595,7 @@ class QuenchPlanAnalysis():
             f = h5py.File(file, 'r')
             self.SIMdata.TimeFrame = np.array(f.get("time_vector"))
             self.SIMdata.I_CoilSections = np.array(f.get("I_CoilSections")).transpose()
-            self.SIMdata.I_CLIQ = np.array(f.get("Ic")) .transpose()
+            self.SIMdata.I_CLIQ = np.array(f.get("Ic")).transpose()
             trigger = np.array(f.get("t_PC"))[0][0]-self.beforePC
             if trigger < float(self.SIMdata.TimeFrame[0]):
                 self.beforePC = -1* self.SIMdata.TimeFrame[0]
@@ -563,6 +605,10 @@ class QuenchPlanAnalysis():
             self.SIMdata.trigger_PC = np.where(abs(self.SIMdata.TimeFrame-np.array(f.get("t_PC"))[0][0])<1E-7)[0][0]
             self.SIMdata.TimeFrame = self.SIMdata.TimeFrame[trigger_shoot:]
             self.SIMdata.I_CoilSections = self.SIMdata.I_CoilSections[:, trigger_shoot:]
+            R_crowbar = np.array(f.get("R_crowbar"))
+            Ud_crowbar = np.array(f.get("Ud_crowbar"))
+            # self.SIMdata.U_PC = self.SIMdata.I_CoilSections*R_crowbar + Ud_crowbar
+            self.SIMdata.U_PC = np.transpose(np.array(f.get("U_PC"))[trigger_shoot:])
             self.SIMdata.I_CLIQ = self.SIMdata.I_CLIQ[trigger_shoot:]
             self.SIMdata.f_CLIQ = self.__FrequencyCLIQ(self.SIMdata.I_CLIQ, SIM=1)
             self.SIMdata.directionsCLIQ = np.array(f.get("directionCurrentCLIQ"))
@@ -586,6 +632,12 @@ class QuenchPlanAnalysis():
             self.SIMdata.U_CoilSections = np.array(f.get("peakUground_half_turns"))
             self.SIMdata.el_connections = np.array(f.get("el_order_half_turns"))
             self.SIMdata.Uturn_half_turns = np.array(f.get("Uturn_half_turns_reordered"))
+            B_field = np.array(f.get("Ba"))
+            strandsToHalfTurn = np.array(f.get("StrandToHalfTurn"))
+            self.SIMdata.B_turns = np.zeros((self.SIMdata.XY_mag_ave.shape[1],))
+            for i in range(len(self.SIMdata.B_turns)):
+                self.SIMdata.B_turns[i] = np.sum(B_field[np.where(strandsToHalfTurn==i+1)[0]])/len(np.where(strandsToHalfTurn==i+1)[0])
+
         else:
             print("Please provide .mat file for Simulation.")
 
@@ -638,24 +690,33 @@ class QuenchPlanAnalysis():
             if channel.name == 'Vdump': self.TDMSdata.U_EE = channel.data
         ## Set up Timeframe
         self.TDMSdata.TimeFrame_MF = np.array([np.linspace(0,self.TDMSdata.t_steps_MF*self.TDMSdata.t_d_MF, int(self.TDMSdata.t_steps_MF))]).transpose()
-        #self.TDMSdata.goupHF['IDCCT_HF'].time_track()
         trigger_shoot = np.gradient(trigger_data, self.TDMSdata.t_d_MF)
         trigger_shoot = np.where(abs(trigger_shoot)>self.TDMSdata.Tolerance)[0][0]-int(self.beforePC/self.TDMSdata.t_d_MF)
         self.TDMSdata.trigger_PC = int(self.beforePC/self.TDMSdata.t_d_MF)
         self.TDMSdata.trigger_shoot = trigger_shoot
-        self.TDMSdata.TimeFrame_MF = self.TDMSdata.TimeFrame_MF[trigger_shoot:, :]-self.TDMSdata.TimeFrame_MF[trigger_shoot, :]\
+        self.TDMSdata.TimeFrame_MF = self.TDMSdata.TimeFrame_MF[self.TDMSdata.trigger_shoot:, :]-self.TDMSdata.TimeFrame_MF[self.TDMSdata.trigger_shoot, :]\
                                      -self.TDMSdata.TimeFrame_MF[self.TDMSdata.trigger_PC+1, :]
-        self.TDMSdata.I_DCCT_MF = self.TDMSdata.I_DCCT_MF[trigger_shoot:, :]
-        self.TDMSdata.U_PC = self.TDMSdata.U_PC[trigger_shoot:, :]
-        self.TDMSdata.I_CLIQ = self.TDMSdata.I_CLIQ[trigger_shoot:, :]
-        self.TDMSdata.U_CLIQ = self.TDMSdata.U_CLIQ[trigger_shoot:, :]
+        self.TDMSdata.I_DCCT_MF = self.TDMSdata.I_DCCT_MF[self.TDMSdata.trigger_shoot:, :]
+        self.TDMSdata.U_PC = self.TDMSdata.U_PC[self.TDMSdata.trigger_shoot:, :]
+        if len(self.TDMSdata.U_PC)<len(self.TDMSdata.I_DCCT_MF):
+            self.TDMSdata.U_PC = np.append(self.TDMSdata.U_PC, 0)
+        self.TDMSdata.I_CLIQ = self.TDMSdata.I_CLIQ[self.TDMSdata.trigger_shoot:, :]
+        self.TDMSdata.U_CLIQ = self.TDMSdata.U_CLIQ[self.TDMSdata.trigger_shoot:, :]
         self.TDMSdata.f_CLIQ = self.__FrequencyCLIQ(self.TDMSdata.I_CLIQ, TDMS= 1)
         if self.TDMSdata.f_CLIQ > 0:
             self.TDMSdata.C_CLIQ = self._CalculateCLIQCapacitance(self.TDMSdata.TimeFrame_MF, self.TDMSdata.I_CLIQ,
                                                                   self.TDMSdata.U_CLIQ, 17)
-        self.TDMSdata.U_EE = self.TDMSdata.U_EE[trigger_shoot-1:]
-        self.__FillHeaterArrays(I_Heaters, trigger_shoot)
-        self.__FillHeaterArrays(U_Heaters, trigger_shoot)
+        self.TDMSdata.U_EE = self.TDMSdata.U_EE[self.TDMSdata.trigger_shoot:]
+        if self.TDMSdata.U_EE.shape[0]>self.TDMSdata.TimeFrame_MF.shape[0]:
+            self.TDMSdata.U_EE = self.TDMSdata.U_EE[self.TDMSdata.TimeFrame_MF.shape[0]-self.TDMSdata.U_EE.shape[0]:]
+        elif self.TDMSdata.TimeFrame_MF.shape[0]>self.TDMSdata.U_EE.shape[0]:
+            self.TDMSdata.U_EE = np.append(self.TDMSdata.U_EE,
+                                           np.array([0]*(self.TDMSdata.TimeFrame_MF.shape[0]-self.TDMSdata.U_EE.shape[0])))
+        if len(I_Heaters)>0:
+            self.__FillHeaterArrays(I_Heaters, self.TDMSdata.trigger_shoot)
+            self.__ClearHeaterOffset()
+        if len(U_Heaters) > 0:
+            self.__FillHeaterArrays(U_Heaters, self.TDMSdata.trigger_shoot)
         self.__CreateVoltageVertices()
         return
 
@@ -670,6 +731,13 @@ class QuenchPlanAnalysis():
             if Heaters[0].startswith('I'): self.TDMSdata.I_QH[order-1] = self.TDMSdata.groupMF[k].data[trigger_shoot:]
             elif Heaters[0].startswith('U'): self.TDMSdata.U_QH[order-1] = self.TDMSdata.groupMF[k].data[trigger_shoot:]
 
+    def __ClearHeaterOffset(self):
+        idx_tQH = np.argmax(self.TDMSdata.I_QH, axis=1)[0] - 2
+        I_QH_offset = np.sum(self.TDMSdata.I_QH[:, :idx_tQH], axis=1) / len(self.TDMSdata.I_QH[0, :idx_tQH])
+        for i in range(self.TDMSdata.I_QH.shape[0]):
+            self.TDMSdata.I_QH[i, :] = self.TDMSdata.I_QH[i, :] - I_QH_offset[i]
+        if self.verbose: print('Cleaned offset in measured I_QH: ', I_QH_offset)
+
     def __CreateVoltageVertices(self):
         ## 1. Extract all channels that are relevant
         # Version 1
@@ -779,6 +847,9 @@ class QuenchPlanAnalysis():
                         key_name = 'C'+row[0]+'_'+row[1]+'_'+row[2]
                     nTurns = float(row[3])
                     if nTurns == 0: continue
+                    if not np.any(self.SIMdata.el_connections):
+                        print('Please first provide eletrical connections from LEDET e.g. by providing a simulation file with the same magnet.')
+                        return
                     turns = self.SIMdata.el_connections[turn_count:turn_count+int(nTurns*2)]
                     current_dict[key_name] = turns[:,0]
                     turn_count = turn_count + int(nTurns*2)
@@ -826,7 +897,8 @@ class QuenchPlanAnalysis():
                             ax.set_xlabel("Time, t [ms]")
                             ax.set_ylabel("Voltage [V]")
                             if idx_tQuench-tSteps<0: tSteps = idx_tQuench
-                            ax.plot(self.TDMSdata.TimeFrame_MF[idx_tQuench-tSteps:idx_tQuench+int(5*tSteps), :]*1000, Udata[Qidx[i]-tSteps:Qidx[i]+int(5*tSteps)],'--')
+                            #ax.plot(self.TDMSdata.TimeFrame_MF[idx_tQuench-tSteps:idx_tQuench+int(5*tSteps), :]*1000, Udata[Qidx[i]-tSteps:Qidx[i]+int(5*tSteps)],'--')
+                            ax.plot(self.TDMSdata.TimeFrame_MF * 1000, Udata[self.TDMSdata.trigger_shoot:], '--')
                             ax.axvline(self.TDMSdata.TimeFrame_MF[idx_tQuench, :]*1000,c='r')
                             ax.axhline(Vthr, c='r', ls='--')
                             ax.legend(["$U_{Res,"+key+"}$","$t_{Quench}$","$U_{Threshold}$"])
@@ -841,7 +913,7 @@ class QuenchPlanAnalysis():
             # ax.set_ylim([-5*Vthr, 5*Vthr])
         if not found:
             print("No quench detected.")
-            found.append(("NaN",0,self.TDMSdata.trigger_PC))
+            found.append(("nan",0,self.TDMSdata.trigger_PC))
         else:
             def getSecond(elem):
                 return elem[1]
@@ -850,7 +922,7 @@ class QuenchPlanAnalysis():
             for j in range(len(found)):
                 print("Detected Quench in {} at {} ms".format(found[j][0], np.round(found[j][1] * 1000, 2)))
 
-            if Plot:
+            if Plot and self.SIMdata.XY_mag_ave.size >0:
                 fig = plt.figure()
                 ax = fig.add_subplot(111)
                 count = 0
@@ -883,26 +955,34 @@ class QuenchPlanAnalysis():
         # Extract R_Circuit
         U_init = sum(self.TDMSdata.U_PC[:self.TDMSdata.trigger_PC, 0])/self.TDMSdata.trigger_PC
         self.TDMSdata.R_circuit = U_init/I_init
+        if self.TDMSdata.R_circuit<0:
+            print("Lead resistance was calculated negative. Please verify, I'll take the absolute value.")
+            self.TDMSdata.R_circuit = abs(self.TDMSdata.R_circuit)
         if self.verbose: print('Measured lead resistance: ', self.TDMSdata.R_circuit)
 
         # Find Ud_crowbar & R_crowbar
         I_th = signal.medfilt(self.TDMSdata.I_DCCT_MF[:, 0], kernel_size = 101)
         I_th[abs(I_th) < I_th_tol] = 0
-        ranges = self.__zero_runs(I_th)[-1][0]
+        ranges = int(self.__zero_runs(I_th)[-1][0])
         idx_add = int(0.05/self.TDMSdata.t_d_MF)
         self.TDMSdata.Ud_crow = abs(np.sum(self.TDMSdata.U_PC[ranges:ranges+idx_add,0])/len(self.TDMSdata.U_PC[ranges:ranges+idx_add,0]))
-        I_crowbar = signal.medfilt(self.TDMSdata.I_DCCT_MF[:,0], kernel_size = 101)
-        I_crow = abs(np.sum(I_crowbar[ranges+idx_add:])/len(I_crowbar[ranges+idx_add:]))
-        U_remaining = abs((np.sum(self.TDMSdata.U_PC[ranges+idx_add:, 0])-self.TDMSdata.Ud_crow)/len(self.TDMSdata.U_PC[ranges+idx_add:,0]))
+        U_PC_res = np.max(abs(self.TDMSdata.U_PC[np.where(self.TDMSdata.U_PC <0)[0][0]:]))
+        idx_PC_res = np.argmax(abs(self.TDMSdata.U_PC[np.where(self.TDMSdata.U_PC <0)[0][0]:]))
+        self.TDMSdata.R_crow = abs(U_PC_res/self.TDMSdata.I_DCCT_MF[idx_PC_res])
         if self.verbose:
-            print('PRELIMINARY (unused): Measured Crowbar resistance: ', U_remaining/I_crow)
+            print('Measured Crowbar resistance: ', self.TDMSdata.R_crow)
             print('Measured Ud_crowbar: ', self.TDMSdata.Ud_crow)
 
         # Find R_EE
-        U_EE = signal.medfilt(self.TDMSdata.U_EE, kernel_size=3)
-        self.TDMSdata.R_EE = np.max(U_EE)/self.TDMSdata.I_DCCT_MF[np.argmax(U_EE)]
-        if self.verbose:
-            print('Measured R_EE: ', self.TDMSdata.R_EE)
+        U_EE = signal.medfilt(self.TDMSdata.U_EE, kernel_size=7)
+        if self.TDMSdata.I_DCCT_MF[np.argmax(U_EE)-1]< 0.01*self.TDMSdata.I_DCCT_MF[0]:
+            print("Current decayed even before tEE. Let's neglect EE.")
+            self.TDMSdata.R_EE = 0
+            self.TDMSdata.t_EE = 9999
+        else:
+            self.TDMSdata.R_EE = np.max(U_EE)/self.TDMSdata.I_DCCT_MF[np.argmax(U_EE)-1]
+            if self.verbose:
+                print('Measured R_EE: ', self.TDMSdata.R_EE)
 
         # R_QH & tau_QH
         self.TDMSdata.R_QH = (np.max(self.TDMSdata.U_QH, axis=1)/np.max(self.TDMSdata.I_QH, axis=1))
@@ -910,9 +990,12 @@ class QuenchPlanAnalysis():
         IQHmax = np.max(self.TDMSdata.I_QH, axis=1)
         IQH_tau = np.exp(-1) * IQHmax
         tTau = np.ones((len(IQH_tau),))
-        if np.any(IQHmax <1):
+        if np.any(IQHmax >1):
             for i in range(len(IQH_tau)):
-                tTau[i] = self.TDMSdata.TimeFrame_MF[np.where(abs(self.TDMSdata.I_QH[i] - IQH_tau[i]) < 1)[0][0]]
+                try:
+                    tTau[i] = self.TDMSdata.TimeFrame_MF[np.where(abs(self.TDMSdata.I_QH[i] - IQH_tau[i]) < 1)[0][0]]
+                except:
+                    tTau[i] = self.TDMSdata.TimeFrame_MF[np.where(abs(self.TDMSdata.I_QH[i] - IQH_tau[i]) < 5)[0][0]]
             tau_QH = tTau - tMax
             self.TDMSdata.tau_QH = tau_QH[0]
             self.TDMSdata.C_QH = self.TDMSdata.tau_QH/ self.TDMSdata.R_QH
@@ -920,7 +1003,6 @@ class QuenchPlanAnalysis():
                 print('Measured Total R_QH: ', self.TDMSdata.R_QH)
                 print('Measured tau_QH: ', self.TDMSdata.tau_QH)
                 print('Measured C_QH: ', self.TDMSdata.C_QH)
-                print('Measured max. U_QH: ', np.max(self.TDMSdata.U_QH,axis=1))
         else:
             self.TDMSdata.t_QH = 0
 
@@ -929,49 +1011,54 @@ class QuenchPlanAnalysis():
 
         return I_init
 
-    def ExtractFeatures(self, SIM= 1, TDMS=1):
+    def ExtractFeatures(self, SIM= 1, TDMS=1, verbose = True):
         ## Extract Timing features:
-        print('Timing')
+        if verbose: print('Timing')
         # # Extract t_PC
         if SIM:
             dIdt_Coil = np.zeros((self.SIMdata.I_CoilSections.shape))
             for i in range((dIdt_Coil.shape[1])):
-                dIdt_Coil[:,i] = np.gradient(self.SIMdata.I_CoilSections[:,i], self.SIMdata.TimeFrame[:,0])
-            t_PC_SIM = self._FindFiring(dIdt_Coil, self.SIMdata.TimeFrame)
-            print('t_PC_Sim: ', t_PC_SIM)
+                dIdt_Coil[:, i] = np.gradient(self.SIMdata.I_CoilSections[:, i], self.SIMdata.TimeFrame[:, 0])
+            t_PC_SIM = np.round(self._FindFiring(dIdt_Coil, self.SIMdata.TimeFrame),5)
+            if verbose: print('t_PC_Sim: ', t_PC_SIM)
         if TDMS:
-            dIdt_Coil = np.array([np.gradient(signal.medfilt(self.TDMSdata.I_DCCT_MF[:,0], kernel_size = 17), self.TDMSdata.TimeFrame_MF[:,0])]).transpose()
-            self.TDMSdata.t_PC = self._FindFiring(dIdt_Coil, self.TDMSdata.TimeFrame_MF, dI_tol=10000)[0]
-            print('t_PC_Meas: ', self.TDMSdata.t_PC)
+            self.TDMSdata.t_PC = np.round(self.TDMSdata.TimeFrame_MF[self.TDMSdata.trigger_PC+1, 0], 7)
+            if verbose: print('t_PC_Meas: ', self.TDMSdata.t_PC)
         # # Extract t_QH
         if SIM:
             dIdt_QH = np.zeros((self.SIMdata.I_QH.shape))
             for i in range((dIdt_QH.shape[0])):
-                dIdt_QH[i,:] = np.gradient(self.SIMdata.I_QH[i,:], self.SIMdata.TimeFrame[:,0])
+                dIdt_QH[i,:] = np.gradient(self.SIMdata.I_QH[i, :], self.SIMdata.TimeFrame[:, 0])
             t_QH_Sim = self._FindFiring(dIdt_QH.transpose(), self.SIMdata.TimeFrame)
-            print('t_QH_Sim: ', t_QH_Sim)
+            if verbose: print('t_QH_Sim: ', t_QH_Sim)
         if TDMS:
-            self.TDMSdata.t_QH = self.TDMSdata.TimeFrame_MF[np.argmax(self.TDMSdata.I_QH, axis=1),0]
-            print('t_QH_Meas: ', np.transpose(self.TDMSdata.t_QH))
+            if np.any(self.TDMSdata.I_QH[:100] > 10):
+                self.TDMSdata.t_QH = self.TDMSdata.TimeFrame_MF[np.argmax(self.TDMSdata.I_QH, axis=1), 0]
+            else:
+                self.TDMSdata.t_QH = [9999]*len(self.TDMSdata.I_QH[:,0])
+            if verbose: print('t_QH_Meas: ', np.transpose(self.TDMSdata.t_QH))
         # # # Extract t_CLIQ
         if SIM:
             dIdt_CLIQ = np.array([np.gradient(self.SIMdata.I_CLIQ[:,0], self.SIMdata.TimeFrame[:,0])]).transpose()
             t_CLIQ_Sim = self._FindFiring(dIdt_CLIQ, self.SIMdata.TimeFrame)
-            print('t_CLIQ_Sim: ', t_CLIQ_Sim)
+            if verbose: print('t_CLIQ_Sim: ', t_CLIQ_Sim)
         if TDMS:
             dIdt_CLIQ = np.array([np.gradient(signal.medfilt(self.TDMSdata.I_CLIQ[:,0], kernel_size = 11), self.TDMSdata.TimeFrame_MF[:,0])]).transpose()
             self.TDMSdata.t_CLIQ = self._FindFiring(dIdt_CLIQ, self.TDMSdata.TimeFrame_MF, dI_tol = 5000)
-            print('t_CLIQ_Meas: ', self.TDMSdata.t_CLIQ)
+            if verbose: print('t_CLIQ_Meas: ', self.TDMSdata.t_CLIQ)
 
         # # # Extract t_EE
         if TDMS:
-            dUdt_EE = np.array([np.gradient(signal.medfilt(self.TDMSdata.U_EE, kernel_size=11),
-                                              self.TDMSdata.TimeFrame_MF[:,0])]).transpose()
-            self.TDMSdata.t_EE = self._FindFiring(dUdt_EE, self.TDMSdata.TimeFrame_MF, dI_tol=5000)
-            print('t_EE: ', self.TDMSdata.t_EE)
+            if np.any(self.TDMSdata.U_EE[:100]>5):
+                dUdt_EE = np.array([np.gradient(signal.medfilt(self.TDMSdata.U_EE, kernel_size=11),
+                                                  self.TDMSdata.TimeFrame_MF[:,0].transpose())]).transpose()
+                self.TDMSdata.t_EE = self._FindFiring(dUdt_EE, self.TDMSdata.TimeFrame_MF, dI_tol=5000)
+            else:
+                self.TDMSdata.t_EE = 9999
+            if verbose: print('t_EE: ', self.TDMSdata.t_EE)
 
         ## Extract t_Quench [if applicable]
-        # self.FindQuenchTDMS(0.1, 0.007)
+        self.FindQuenchTDMS(0.1, 0.007)
 
         ## Extract electrical parameters
         I_init = self.__ExtractElectricalParameters()
