@@ -90,16 +90,16 @@ def CompleteRun():
     # Sw.prepareSimulation(MagnetName, "C:\\cernbox\\TEMP2\\", OffsetNumber= 42, ROXIE_File='C:\\cernbox\\SWAN_projects\\steam-notebooks\\steam-ledet-input\\MCBXV\\MCBXV_CopperWedges_All_WithIron_WithSelfField.map2d')
     # Sw.prepareSimulation(MagnetName, "C:\\cernbox\\TEMP2\\", OffsetNumber=17, ROXIE_File='C:\\cernbox\\SWAN_projects\\steam-notebooks\\steam-ledet-input\\MCBY_1AP\\MCBY_1AP_CopperWedges_ThCool_All_WithIron_WithSelfField.map2d')
     # Sw.prepareSimulation(MagnetName, "C:\\cernbox\\TEMP\\", OffsetNumber=50)
-    Sw.prepareSimulation(MagnetName, "C:\\cernbox\\TEMP\\")
+    # Sw.prepareSimulation(MagnetName, "C:\\cernbox\\TEMP\\")
 
 
     end = time.time()
     print("Time:", end - start)
     print("Preparation done")
 
-    SimNumbers = np.linspace(1,216, 216).astype(int)
+    SimNumbers = np.linspace(0,215, 216).astype(int)
 
-    RunSimulations(ledetFolder, ledetExe, MagnetName, Simulations = SimNumbers, RunSimulations=False)
+    # RunSimulations(ledetFolder, ledetExe, MagnetName, Simulations = SimNumbers, RunSimulations=False)
     # EvaluateSimulations("C:\\cernbox\\LEDET\\LEDET\\MCO\\Output\\Txt Files", 'MCO',
     #                     'C:\\cernbox\\Validation_MCDO\\Exp Data\\RCO.A56B1_2018-03-16_PM_I_A_AutoAlign.csv',
     #                     Sw, Mat=False, SkipAlign=True)
@@ -134,6 +134,10 @@ def CompleteRun():
     #                     MagnetName,
     #                     'C:\\cernbox\\MasterThesis\\Test23_ParSweepMeas.csv',
     #                     Sw, Mat=False)
+    EvaluateSimulations("C:\\cernbox\\LEDET_OS\\LEDET\\LEDET\\MQXFS4b\\Output\\Txt Files",
+                        MagnetName,
+                        'C:\\cernbox\\MasterThesis\\Measurement_data\\Signal_I_CLIQTest7.csv',
+                        Sw, Mat=False)
 
 def AutomaticRun():
     LEDETFolder = 'C:\\cernbox\LEDET\\LEDET_SimulationFiles_Quench'
@@ -158,14 +162,20 @@ def testConsistencyChecks():
     print(x)
 
 def testvQ():
-    nameFileLEDET = "C:\cernbox\SWAN_projects\steam-notebooks\steam-ledet-input\MQXA\MQXA_0.xlsx"
+    nameFileLEDET = "C:\cernbox\SWAN_projects\steam-notebooks\steam-ledet-input\MQM_2in1\MQM_2in1_0.xlsx"
+    RoxieFile = "C:\\cernbox\\SWAN_projects\\steam-notebooks\\steam-ledet-input\\MQM_2in1\\FieldMaps\\"
+    # nameFileLEDET = "C:\cernbox\SWAN_projects\steam-notebooks\steam-ledet-input\MQXA\MQXA_0.xlsx"
     # nameFileLEDET = 'C:\cernbox\LEDET_MT\LEDET\MQXFBP2\Input\MQXFBP2_10.xlsx'
-    RoxieFile = "C:\cernbox\SWAN_projects\steam-notebooks\steam-ledet-input\MQXA\MQXA_All_WithIron_WithSelfField.map2d"
+    # RoxieFile = "C:\cernbox\SWAN_projects\steam-notebooks\steam-ledet-input\MQXA\MQXA_All_WithIron_WithSelfField.map2d"
     # RoxieFile = "C:\cernbox\LEDET_MT\Field Maps\MQXFBP2\MQXFBP2_All_WithIron_WithSelfField.map2d"
     # RoxieFile = "C:\\cernbox\\LEDET_MT\\Field Maps\\MQXFBP2\\"
     a = ParametersLEDET()
     a.readLEDETExcel(nameFileLEDET)
-    l = a.adjust_vQ(RoxieFile, Transversaldelay = 'Long')
+    # l = a.adjust_vQ(RoxieFile, Transversaldelay = 'Long')
+    # a.adjust_vQ(RoxieFile, CurrentsInCoilsections=[5141, 3000])
+    a.adjust_vQ(RoxieFile)
+    [_, th_con_h, th_con_w] = a._obtainThermalConnections()
+    a.adjust_vQ_QuenchHeater(th_con_h, th_con_w, 12)
     return a
 
 def testQuenchPlanAnalysis(FileName_TDMS, FileName_Sim, verbose = False):
@@ -548,9 +558,11 @@ def testMP3():
 # td_data = "C:\\cernbox\\MasterThesis\\Measurement_data\\MQXFS4b\\HCMQXSM001-CR000042__H1810171007_a036(0).tdms"
 # td_data = "C:\\cernbox\\MasterThesis\\Measurement_data\\MQXFS4b\\HCMQXSM001-CR000042__H1810171144_a039(0).tdms"
 # td_data = "C:\\cernbox\\MasterThesis\\Measurement_data\\MQXFS4b\\HCMQXSM001-CR000042__H1810161600_a033(0).tdms"
-
+td_data = "C:\\cernbox\\MasterThesis\\Measurement_data\\MQXFS4b\\HCMQXSM001-CR000042__H1810111528_a005(0).tdms"
 # td_data = "C:\\cernbox\\MasterThesis\\Measurement_data\\MQXFS4b\\HCMQXSM001-CR000042__H1810121620_a019(0).tdms"
-td_data = "C:\\cernbox\\MasterThesis\\Measurement_data\\MQXFS4b\\HCMQXSM001-CR000042__H1810101443_tb010(0).tdms"
+# td_data = "C:\\cernbox\\MasterThesis\\Measurement_data\\MQXFS4b\\HCMQXSM001-CR000042__H1810101443_tb010(0).tdms"
+# td_data = "C:\\cernbox\\MasterThesis\\Measurement_data\\MQXFS4b\\HCMQXSM001-CR000042__H1810111630_a007(0).tdms"
+# td_data = "C:\\cernbox\\MasterThesis\\Measurement_data\\MQXFS4b\\HCMQXSM001-CR000042__H1810091620_ta001(0).tdms"
 
 # sim_data = "C:\\cernbox\\LEDET_MT\\LEDET\\MQXFBP2\\Output\\Mat Files\\SimulationResults_LEDET_10.mat"
 # sim_data = "C:\\cernbox\\LEDET_MT\\LEDET\\MQXFS4b\\Output\\Mat Files\\SimulationResults_LEDET_12.mat"
@@ -562,19 +574,31 @@ td_data = "C:\\cernbox\\MasterThesis\\Measurement_data\\MQXFS4b\\HCMQXSM001-CR00
 # sim_data = "C:\\cernbox\\LEDET_MT\\LEDET\\MQXFS4b\\Output\\Mat Files\\SimulationResults_LEDET_3.mat"
 # sim_data = "C:\\cernbox\\LEDET_MT\\LEDET\\MQXFS4b\\Output\\Mat Files\\SimulationResults_LEDET_12.mat"
 # sim_data = "C:\\cernbox\\LEDET_MT\\LEDET\\MQXFS4b\\Output\\Mat Files\\SimulationResults_LEDET_26.mat"
-# sim_data = "C:\\cernbox\\LEDET\\LEDET_MT\\LEDET\\MQXFS4b\\Output\\Mat Files\\SimulationResults_LEDET_44.mat"
-
-# Ref_File = "C:\\cernbox\\MasterThesis\\MQXFS4b_Ref_0.xlsx"
+# sim_data = "C:\\cernbox\\LEDET\\LEDET_MT\\LEDET\\MQXFS4b\\Output\\Mat Files\\SimulationResults_LEDET_28.mat"
+# # #
+Ref_File = "C:\\cernbox\\MasterThesis\\MQXFS4b_Ref_0.xlsx"
 OutputDirectory = "C:\\cernbox\\TEMP_toDelete"
+CoilCsv = 'C:\\cernbox\\MasterThesis\\CoilStructure_MQXFS4b.csv'
+Roxie = "C:\\cernbox\\LEDET\\LEDET_MT\\Field maps\\MQXFS4b\\"
+
+a = testQuenchPlanAnalysis(td_data, 0, verbose=True)
+a.PrepareSimulation_FromMeasurements(Ref_File, OutputDirectory, CoilCsv = CoilCsv, nQHseries = 2, ROXIE= Roxie)
+# a.ProvideTurnsToCoilStructure(CoilCsv=CoilCsv)
 
-a = testQuenchPlanAnalysis(td_data, 0)
 # a.SaveSignalToCSV("I_CLIQ",OutputDirectory=OutputDirectory)
-# a.PrepareSimulation_FromMeasurements(Ref_File, OutputDirectory, nQHseries = 2)
 
-a.ExtractFeatures(SIM=0)
+# a.ExtractFeatures(SIM=0)
 # a = testQuenchPlanAnalysis(0, sim_data)
 # a = testQuenchPlanAnalysis(td_data, 0)
 
+# file = "C:\\cernbox\\LEDET\\LEDET_MT\\LEDET\\MQXFS4b\\Input\\MQXFS4b_28.xlsx"
+# Roxie = "C:\\cernbox\\LEDET\\LEDET_MT\\Field maps\\MQXFS4b\\"
+# a = ParametersLEDET()
+# a.readLEDETExcel(file)
+# a.adjust_vQ(Roxie)
+
+
+
 
 # a = ParametersLEDET()
 # a.readLEDETExcel(testfile7)
