@@ -400,11 +400,10 @@ class ParametersCOSIM:
                         if i != SplitCount - SectionCount: continue
                         if i>lenCSections[l]-1: continue
                         SplitCount = SplitCount + 1
-                        if SplitCount == len(CSections):
-                            i = SplitCount-1
+                        if SplitCount == len(CSections): i = SplitCount-1
                     struct = {}
-                    if SplitCount == len(CSections):
-                        struct["name"] = "Port_" + str(i + 1) + "_" + str(0)
+                    if split:
+                        struct["name"] = "Port_" + str(SplitCount) + "_" + str(0)
                     else:
                         struct["name"] = "Port_" + str(i+l*len(CoilSections)+1) + "_" + str(0)
                     struct["components"] = []
@@ -463,6 +462,7 @@ class ParametersCOSIM:
             pass
 
         struct_total = []
+        split_idx = 0
         for n in range(0, N_LEDET):
             if n+1== QuenchMagnet and QuenchMagnet != 0:
                 quench = '_Quench'
@@ -483,15 +483,23 @@ class ParametersCOSIM:
                 DM = ''
             else: convEl = convergenceElement[:-1]
 
+
             for i in range(len(CoilSections)):  # Number of Ports
                 if split and len(CoilSections[n]) != len(CoilSections):
-                    if i+1 != n: continue
+                    if i+1 != n and len(CoilSections[n])>1: continue
+                    if i>=len(CoilSections[n]): continue
+                if split:
+                    split_idx = split_idx + 1
                 for j in range(2):  # 2Ports in PSpice
                     struct = {}
-                    struct["name"] = "Port_" + str(i+n*len(CoilSections)+1) + "_" + str(j)
-                    if CombinedModel: LSections[i] = Components[0] + "_" + str(i+n*len(CoilSections)+1)
+                    if split:
+                        struct["name"] = "Port_" + str(split_idx) + "_" + str(j)
+                        if CombinedModel: LSections[i] = Components[0] + "_" + str(split_idx)
+                    else:
+                        struct["name"] = "Port_" + str(i+n*len(CoilSections)+1) + "_" + str(j)
+                        if CombinedModel: LSections[i] = Components[0] + "_" + str(i+n*len(CoilSections)+1)
                     struct["components"] = [LSections[i]]
-                    Number = LSections[i][-1]
+                    Number = [int(s) for s in LSections[i].split('_') if s.isdigit()][0]
                     struct["inputs"] = []
                     for l in range(len(couplingParameterPSpice[j][0])):  # Number of In's
                         substruct_in = {}
