@@ -8,6 +8,7 @@ import numpy as np
 import yaml
 import shutil
 import json
+import time
 from steam_nb_api.ledet.ParameterSweep import *
 
 def _read_yaml(type_str, elem_name):
@@ -55,7 +56,7 @@ class LibSim_setup:
                                                     [self.Options.executeCleanRun.tolist()[-1]] * (N_LEDET-1))
         return
 
-    def __init__(self, circuit, ParameterFile, Opts='Default', enableQuench=0):
+    def __init__(self, circuit, ParameterFile, Opts='Default'):
         # Folder and Executables
         self.path_PSPICELib = ''
         self.path_NotebookLib = ''
@@ -66,11 +67,13 @@ class LibSim_setup:
         self.EOS_stub_C = ''
         self.EOS_stub_EOS = ''
         self.PSPICE_Folder = ''
+        self.Simulation_Folder = ''
 
         self.ParameterFile = ParameterFile
 
         self.circuit = circuit
         self.transient = ''
+        self.enableQuench = 0
 
         if Opts == 'Default':
             self.Options = Options()
@@ -78,15 +81,13 @@ class LibSim_setup:
         elif Opts == 'Param_Table':
             parameter_dict = self._loadParameterFile()
             self.Options = Options()
-            options_attrs = [a for a in dir(self.Options) if not a.startswith('__')]
-            [setattr(self.Options, op_atr, json.loads(parameter_dict[op_atr])) for op_atr in options_attrs]
+            [setattr(self.Options, op_atr, parameter_dict[op_atr]) for op_atr in self.Options.__annotations__]
             self.flag_Options = 0
             self.__checkOptions()
         else:
             self.flag_Options = 0
             self.Options = Opts
             self.__checkOptions()
-        self.enableQuench = enableQuench
         self.QuenchMagnet = 0
         self.ManualStimuli = []
         return
@@ -115,6 +116,10 @@ class LibSim_setup:
             self.PSPICE_Folder = config_dict['PSPICE_Folder']
         except:
             self.PSPICE_Folder = ''
+        try:
+            self.Change_Folder = config_dict['Change_Folder']
+        except:
+            self.Change_Folder = ''
 
         self.EOS_stub_EOS = EOS_stub_EOS
         if os.getcwd().startswith('C'):
@@ -143,26 +148,40 @@ class LibSim_setup:
                 try:
                     parameter_dict[row_parameter[i]] = float(values[i])
                 except:
-                    parameter_dict[row_parameter[i]] = values[i]
+                    try:
+                        if row_parameter[i] == 'MagnetName' or row_parameter[i] == 'ManualCircuit':
+                            parameter_dict[row_parameter[i]] = json.loads(values[i])
+                        else:
+                            parameter_dict[row_parameter[i]] = json.loads(values[i].lower())
+                    except:
+                        parameter_dict[row_parameter[i]] = values[i]
         return parameter_dict
 
-    def SetUpSimulation(self, transient, Current='', Append: bool = False, ManualCircuit: str = '', ManualStimuli: list = [], AppendStimuli = '', HierCOSIM = '', convergenceElement = '', CombinedModel = False):
+    def SetUpSimulation(self, transient, Current='', Append: bool = False, ManualCircuit: str = '', ManualStimuli: list = [],
+                        AppendStimuli = '', HierCOSIM = '', convergenceElement = '', CombinedModel = False):
+        with open(os.path.join(self.ModelFolder_EOS, 'InSetUp.txt'), 'a') as the_file:
+            the_file.write('Temp file to mark running set-up.\n')
+        time.sleep(30)
+
         self.transient = transient
         parameter_dict = self._loadParameterFile()
 
         if ManualStimuli:
             self.ManualStimuli = ManualStimuli
+        if 'Quench' in parameter_dict.keys():
+            self.enableQuench = parameter_dict['Quench']
+        else:
+            self.enableQuench = 0
 
         if Current == 'Param_Table':
-            Current = json.loads(parameter_dict['Current'])
+            Current = parameter_dict['Current']
         if ManualCircuit == 'Param_Table':
-            string = parameter_dict['ManualCircuit']
-            ManualCircuit = json.loads(string)
+            ManualCircuit = parameter_dict['ManualCircuit']
             ManualCircuit = os.path.join(self.path_NotebookLib, *["steam-sing-input"], *ManualCircuit)
         if convergenceElement == 'Param_Table':
             convergenceElement = parameter_dict['convergenceElement']
         if CombinedModel == 'Param_Table':
-            CombinedModel = json.loads(parameter_dict['CombinedModel'].lower())
+            CombinedModel = parameter_dict['CombinedModel']
 
         if not isinstance(Current, list):
             Current = [Current]
@@ -181,7 +200,7 @@ class LibSim_setup:
 
             if parameter_dict['NumberOfMagnets']>1:
                 try:
-                    MagnetName = json.loads(parameter_dict['MagnetName'])
+                    MagnetName = parameter_dict['MagnetName']
                     N_LEDET = len(MagnetName)
                 except:
                     N_LEDET = 2
@@ -192,7 +211,7 @@ class LibSim_setup:
             Sim_Flag = parameter_dict['flag_COSIM']
             if Sim_Flag:
                 if self.flag_Options and i==0: self.__autoConstructOptions(N_LEDET)
-                CoilSections = json.loads(parameter_dict['CoilSections'])
+                CoilSections = parameter_dict['CoilSections']
                 if not ManualCircuit: circuit_file = self.__findCircuit(self.circuit)
                 else: circuit_file = ManualCircuit
                 COSIMfolder = self._setUp_COSIM(N_LEDET, CoilSections, circuit_file, parameter_dict, AppendStimuli = AppendStimuli,
@@ -208,6 +227,8 @@ class LibSim_setup:
         else:
             newModelCosim.writeCOSIMBatch(Folder, self.CosimExecutable,
                                           Destination=self.ModelFolder_EOS, LEDET_exe=self.LedetExecutable, Append = Append)
+        time.sleep(30)
+        os.remove(os.path.join(self.ModelFolder_EOS, 'InSetUp.txt'))
 
     def StampBatch(self, Stamp, move_to_mod_fol=False):
         newModelCosim = ParametersCOSIM('1', '1', '1')
@@ -383,13 +404,14 @@ class LibSim_setup:
 
         return Stimuli
 
-    def _setUp_COSIM(self, N_LEDET, CoilSections, circuit, parameter_dict, AppendStimuli = '', HierCOSIM = '', convergenceElement='', CombinedModel = False):
+    def _setUp_COSIM(self, N_LEDET, CoilSections, circuit, parameter_dict, AppendStimuli = '', HierCOSIM = '',
+                     convergenceElement='', CombinedModel = False):
         current_level = parameter_dict['I_00']
-        try:
-            MagnetName = json.loads(parameter_dict['MagnetName'])
+        MagnetName = parameter_dict['MagnetName']
+
+        if isinstance(parameter_dict['MagnetName'],list):
             DistinctMagnets = len(MagnetName)
-        except:
-            MagnetName = parameter_dict['MagnetName']
+        else:
             DistinctMagnets = 1
 
         if self.enableQuench and not HierCOSIM:
@@ -405,15 +427,24 @@ class LibSim_setup:
         usualTransients = ['FPA','unbalancedFPA']
         if not self.transient in usualTransients: cl = ''
 
-        if not os.path.isdir(os.path.join(self.ModelFolder_EOS, 'COSIM_'+self.circuit)):
-            os.mkdir(os.path.join(self.ModelFolder_EOS, 'COSIM_'+self.circuit))
-        if not os.path.isdir(os.path.join(self.ModelFolder_EOS, 'COSIM_'+self.circuit, self.transient + str(cl) +quenchTitle +HierCOSIM)):
-            os.mkdir(os.path.join(self.ModelFolder_EOS, 'COSIM_'+self.circuit, self.transient + str(cl) +quenchTitle +HierCOSIM))
-        if not os.path.isdir(os.path.join(self.ModelFolder_EOS, 'COSIM_'+self.circuit, self.transient + str(cl) +quenchTitle +HierCOSIM,'Input')):
-            os.mkdir(os.path.join(self.ModelFolder_EOS, 'COSIM_'+self.circuit, self.transient + str(cl) +quenchTitle +HierCOSIM,'Input'))
-        ModelFolder_EOS = os.path.join(self.ModelFolder_EOS, 'COSIM_'+self.circuit, self.transient + str(cl) +quenchTitle +HierCOSIM,'Input')
-        ModelFolder_C =  self.Folder + '\\COSIM_'+self.circuit +'\\' + self.transient + str(cl) +quenchTitle +HierCOSIM + '\\Input\\'
-        ResultFolder_C = self.Folder + '\\COSIM_'+self.circuit +'\\' + self.transient + str(cl) +quenchTitle +HierCOSIM + '\\Output\\'
+        naming_transient = ''
+        if self.transient.count('_')>2:
+            naming_transient = '_'+ self.transient[:17]
+
+        self.Simulation_Folder = os.path.join(self.ModelFolder_EOS, 'COSIM_'+self.circuit+ naming_transient)
+        if not os.path.isdir(os.path.join(self.ModelFolder_EOS, 'COSIM_'+self.circuit+ naming_transient)):
+            os.mkdir(os.path.join(self.ModelFolder_EOS, 'COSIM_'+self.circuit+ naming_transient))
+        if not os.path.isdir(os.path.join(self.ModelFolder_EOS, 'COSIM_'+self.circuit+ naming_transient, self.transient + str(cl) +quenchTitle +HierCOSIM)):
+            os.mkdir(os.path.join(self.ModelFolder_EOS, 'COSIM_'+self.circuit+ naming_transient, self.transient + str(cl) +quenchTitle +HierCOSIM))
+        if not os.path.isdir(os.path.join(self.ModelFolder_EOS, 'COSIM_'+self.circuit+ naming_transient, self.transient + str(cl) +quenchTitle +HierCOSIM,'Input')):
+            os.mkdir(os.path.join(self.ModelFolder_EOS, 'COSIM_'+self.circuit+ naming_transient, self.transient + str(cl) +quenchTitle +HierCOSIM,'Input'))
+        ModelFolder_EOS = os.path.join(self.ModelFolder_EOS, 'COSIM_'+self.circuit+ naming_transient, self.transient + str(cl) +quenchTitle +HierCOSIM,'Input')
+        if not self.Change_Folder:
+            ModelFolder_C =  self.Folder + '\\COSIM_'+self.circuit+ naming_transient +'\\' + self.transient + str(cl) +quenchTitle +HierCOSIM + '\\Input\\'
+            ResultFolder_C = self.Folder + '\\COSIM_'+self.circuit+ naming_transient +'\\' + self.transient + str(cl) +quenchTitle +HierCOSIM + '\\Output\\'
+        else:
+            ModelFolder_C = self.Change_Folder + '\\COSIM_'  + self.circuit+ naming_transient + '\\' + self.transient + str(cl) + quenchTitle + HierCOSIM + '\\Input\\'
+            ResultFolder_C = self.Change_Folder + '\\COSIM_' + self.circuit+ naming_transient + '\\' + self.transient + str(cl) + quenchTitle + HierCOSIM + '\\Output\\'
         COSIMfolder = ModelFolder_C
 
         if len(MagnetName)!= N_LEDET: Magnets = [MagnetName] * N_LEDET
@@ -484,15 +515,41 @@ class LibSim_setup:
         LEDETfiles = newModelCosim.copyCOSIMfiles(new_circuit, StimulusFile, Magnets, N_LEDET=N_LEDET, ManuallyStimulusFile = StimulusFile)
         prepareLEDETFiles = newModelCosim.prepareLEDETFiles(LEDETfiles, N_PAR=N_LEDET)
 
+        idx = 0
         for file in prepareLEDETFiles:
             par_dict = deepcopy(parameter_dict)
             if self.transient == 'unbalancedFPA':
+                if len(CoilSections)>1:
+                    par_dict['I00'] = parameter_dict['I_00']
+                else:
+                    del par_dict['I00']
+            else:
                 del par_dict['I00']
+
+            if 'i_qT' in parameter_dict.keys():
+                if isinstance(parameter_dict['i_qT'], list):
+                    if isinstance(parameter_dict['i_qT'][idx], list):
+                        par_dict['i_qT'] = parameter_dict['i_qT'][idx]
+                    elif idx == 0:
+                        par_dict['i_qT'] = parameter_dict['i_qT']
+                    else:
+                        self.enableQuench = 0
+                elif idx > 0:
+                    self.enableQuench = 0
+            if 'tStartQuench' in parameter_dict.keys():
+                if isinstance(parameter_dict['tStartQuench'], list):
+                    if isinstance(parameter_dict['tStartQuench'][idx], list):
+                        par_dict['tStartQuench'] = parameter_dict['tStartQuench'][idx]
+                    elif idx == 0:
+                        par_dict['tStartQuench'] = parameter_dict['tStartQuench']
+                    else:
+                        self.enableQuench = 0
+                elif idx > 0:
+                    self.enableQuench = 0
             self._manipulateLEDETExcel(file, par_dict)
-            if len(prepareLEDETFiles) > 1: self.enableQuench = 0
+            idx = idx + 1
         currentDT = datetime.datetime.now()
-        print('COSIM_model_' + self.circuit + "_" + self.transient +
-                                            str(cl)+HierCOSIM + " generated.")
+        print('COSIM_model_' + self.circuit + "_" + self.transient + str(cl)+HierCOSIM + " generated.")
         print('Time stamp: ' + str(currentDT))
         print(' ')
         return COSIMfolder
@@ -519,13 +576,21 @@ class LibSim_setup:
         if HierCOSIM: cl = ''
         else: cl = "_" + str(current_level) + "A"
 
-        if not os.path.isdir(os.path.join(self.ModelFolder_EOS, 'LEDET_model_' + self.circuit)):
-            os.mkdir(os.path.join(self.ModelFolder_EOS, 'LEDET_model_' + self.circuit))
-        if not os.path.isdir(os.path.join(self.ModelFolder_EOS, 'LEDET_model_' + self.circuit, self.transient + cl+ HierCOSIM )):
-            os.mkdir(os.path.join(self.ModelFolder_EOS, 'LEDET_model_' + self.circuit, self.transient + cl + HierCOSIM ))
-        ModelFolder_EOS = os.path.join(self.ModelFolder_EOS, 'LEDET_model_' + self.circuit, self.transient + cl + HierCOSIM)
+        naming_transient = ''
+        if self.transient.count('_')>2:
+            naming_transient = '_'+self.transient[:17]
 
-        ModelFolder_C = os.path.join(self.Folder + 'LEDET_model_' + self.circuit, self.transient + cl + HierCOSIM)
+        self.Simulation_Folder = os.path.join(self.ModelFolder_EOS, 'LEDET_model_' + self.circuit+ naming_transient )
+        if not os.path.isdir(os.path.join(self.ModelFolder_EOS, 'LEDET_model_' + self.circuit+ naming_transient )):
+            os.mkdir(os.path.join(self.ModelFolder_EOS, 'LEDET_model_' + self.circuit+ naming_transient))
+        if not os.path.isdir(os.path.join(self.ModelFolder_EOS, 'LEDET_model_' + self.circuit+ naming_transient, self.transient + cl+ HierCOSIM )):
+            os.mkdir(os.path.join(self.ModelFolder_EOS, 'LEDET_model_' + self.circuit+ naming_transient, self.transient + cl + HierCOSIM ))
+        ModelFolder_EOS = os.path.join(self.ModelFolder_EOS, 'LEDET_model_' + self.circuit+ naming_transient, self.transient + cl + HierCOSIM)
+
+        if not self.Change_Folder:
+            ModelFolder_C = os.path.join(self.Folder + 'LEDET_model_' + self.circuit + naming_transient, self.transient + cl + HierCOSIM)
+        else:
+            ModelFolder_C = os.path.join(self.Change_Folder + 'LEDET_model_' + self.circuit + naming_transient, self.transient + cl + HierCOSIM)
         COSIMfolder = str(MagnetName) + '_L_' + ModelFolder_C
         newModelCosim = ParametersCOSIM(ModelFolder_EOS, nameMagnet=MagnetName, nameCircuit=self.circuit)
         newModelCosim.makeAllFolders(N_LEDET=1, LEDET_only=1)
@@ -539,7 +604,7 @@ class LibSim_setup:
             parameter_dict["flag_generateReport"] = 1
         if not "flag_saveTxtFiles" in parameter_dict.keys():
             parameter_dict["flag_saveTxtFiles"] = 1
-        self._manipulateLEDETExcel(nameFileLEDET, parameter_dict)
+        self._manipulateLEDETExcel(nameFileLEDET, parameter_dict, LEDET=1)
 
         # Display time stamp and end run
         currentDT = datetime.datetime.now()
@@ -571,15 +636,21 @@ class LibSim_setup:
             time_vector = time_vector + [self.Options.t_0[i]+timeSteps[i], timeSteps[i], self.Options.t_end[i]]
         return time_vector
 
-    def _manipulateLEDETExcel(self, file, parameter_dict):
+    def _manipulateLEDETExcel(self, file, parameter_dict, LEDET=0):
         PL = ParametersLEDET()
         PL.readLEDETExcel(file, verbose = False)
         time_vector = self.__generateTimeVectorLEDET()
         PL.Options.time_vector_params = time_vector
         PL.Inputs.tQuench = [self.Options.t_0[0]]*len(PL.Inputs.tQuench)
 
+        CurrentsInCoilsections = []
+        if 'I00' in parameter_dict.keys() and not LEDET:
+            CurrentsInCoilsections = parameter_dict['I00']
+            del parameter_dict['I00']
+
         for key in parameter_dict.keys():
             if key in PL.Inputs.__annotations__:
+                if key == 'tStartQuench': continue
                 if not isinstance(parameter_dict[key], list):
                     values = self.__generateImitate(PL, PL.getAttribute(getattr(PL, 'Inputs'), key), parameter_dict[key])
                 elif len(parameter_dict[key])==1:
@@ -589,41 +660,83 @@ class LibSim_setup:
             if key in PL.Options.__annotations__:
                 PL.setAttribute("Options", key, parameter_dict[key])
 
+        MagName1 = os.path.abspath(os.path.join(os.path.abspath(os.path.join(file, os.pardir)), os.pardir))
+        MagName2 = os.path.abspath(os.path.join(MagName1, os.pardir))
+        MagName = MagName1.replace(MagName2 + '/', '')
+        fieldmaps = os.path.join(self.path_NotebookLib, 'steam-ledet-input', MagName)
         if self.enableQuench:
-            if parameter_dict['Quench']:
-                MagName1 = os.path.abspath(os.path.join(os.path.abspath(os.path.join(file, os.pardir)), os.pardir))
-                MagName2 = os.path.abspath(os.path.join(MagName1, os.pardir))
-                MagName = MagName1.replace(MagName2+'/','')
-                fieldmaps = os.path.join(self.path_NotebookLib, 'steam-ledet-input', MagName)
+            calc_tQD = 0
+            if 'tStartQuench' not in parameter_dict.keys(): calc_tQD = 1
+            elif isinstance(parameter_dict['tStartQuench'],list):
+                if isinstance(parameter_dict['tStartQuench'][0], bool): calc_tQD = 1
+            if calc_tQD:
                 [l, tQD] = PL.adjust_vQ(fieldmaps, Transversaldelay= 'Short')
 
                 if tQD < 0.01:
                     print("Increased detection margin to 10 ms.")
                     tQD = 0.01
-                i_qT = int(parameter_dict['i_qT'])
-                t_Q = parameter_dict['t_PC']-tQD
+                tQD = parameter_dict['t_PC'] - tQD
+                tQD = [tQD]*len(parameter_dict['i_qT'])
+            else:
+                tQD = parameter_dict['tStartQuench']
+                if not isinstance(tQD, list): tQD = [tQD]
+                tQD = [float(i) for i in tQD]
+            if not isinstance(tQD, list): tQD = [tQD]
+            i_qT = parameter_dict['i_qT']
+            if not isinstance(i_qT, list): i_qT = [i_qT]
+
+            if len(PL.Inputs.iStartQuench) > 1:
+                tStartQuench = [9999] * len(PL.Inputs.tStartQuench)
+                iStartQuench = np.linspace(1, len(PL.Inputs.vQ_iStartQuench), len(PL.Inputs.vQ_iStartQuench))
+                lengthHotSpot_iStartQuench = [0.01] * len(PL.Inputs.vQ_iStartQuench)
+                if len(CurrentsInCoilsections)>0:
+                    PL.adjust_vQ(fieldmaps, CurrentsInCoilsections=CurrentsInCoilsections)
+                vQ_iStartQuench = PL.Inputs.vQ_iStartQuench
+            else:
+                iStartQuench = []
+                tStartQuench = []
+                lengthHotSpot_iStartQuench = []
+                vQ_iStartQuench = []
+
+            for i in range(len(tQD)):
+                t_Q = tQD[i]
                 if t_Q<-5: t_Q = -5
                 if len(PL.Inputs.iStartQuench)>1:
-                    tStartQuench = [9999]*len(PL.Inputs.tStartQuench)
-                    tStartQuench[i_qT - 1] = t_Q
-                    iStartQuench = np.linspace(1,len(PL.Inputs.vQ_iStartQuench),len(PL.Inputs.vQ_iStartQuench))
+                    tStartQuench[int(i_qT[i]) - 1] = t_Q
                     lengthHotSpot_iStartQuench = [0.01] * len(PL.Inputs.vQ_iStartQuench)
-                    vQ_iStartQuench= PL.Inputs.vQ_iStartQuench
-                    vQ_iStartQuench[i_qT - 1]= vQ_iStartQuench[i_qT - 1] * 2
+                    vQ_iStartQuench[int(i_qT[i]) - 1]= vQ_iStartQuench[int(i_qT[i])  - 1] * 2
                 else:
-                    iStartQuench = [i_qT]
-                    tStartQuench = [t_Q]
-                    lengthHotSpot_iStartQuench = [0.01]
-                    vQ_iStartQuench = PL.Inputs.vQ_iStartQuench[i_qT -1]*2
+                    iStartQuench.append(int(i_qT[i]))
+                    tStartQuench.append(t_Q)
+                    lengthHotSpot_iStartQuench.append(0.01)
+                    vQ_iStartQuench.append(PL.Inputs.vQ_iStartQuench[int(i_qT[i]) -1]*2)
 
-                PL.setAttribute("Inputs", "iStartQuench", iStartQuench)
-                PL.setAttribute("Inputs", "lengthHotSpot_iStartQuench", lengthHotSpot_iStartQuench)
-                PL.setAttribute("Inputs", "vQ_iStartQuench", vQ_iStartQuench)
-                PL.Inputs.tQuench = [t_Q] * len(PL.Inputs.tQuench)
+            PL.setAttribute("Inputs", "iStartQuench", iStartQuench)
+            PL.setAttribute("Inputs", "lengthHotSpot_iStartQuench", lengthHotSpot_iStartQuench)
+            PL.setAttribute("Inputs", "vQ_iStartQuench", vQ_iStartQuench)
+            PL.Inputs.tQuench = [min(tQD)] * len(PL.Inputs.tQuench)
         else:
             tStartQuench = [9999] * len(PL.Inputs.tStartQuench)
+            if len(CurrentsInCoilsections) > 0:
+                PL.adjust_vQ(fieldmaps, CurrentsInCoilsections=CurrentsInCoilsections)
+                PL.setAttribute("Inputs", "iStartQuench", np.linspace(1,len(PL.Inputs.vQ_iStartQuench),len(PL.Inputs.vQ_iStartQuench)))
+                PL.setAttribute("Inputs", "lengthHotSpot_iStartQuench", [0.01]*len(PL.Inputs.vQ_iStartQuench))
+                tStartQuench = [9999] * len(PL.Inputs.vQ_iStartQuench)
+
+                HalfTurnToGroup = np.repeat(PL.Inputs.GroupToCoilSection, PL.Inputs.nT.astype(int))
+                lastj = 0
+                for j in PL.Inputs.el_order_half_turns:
+                    if HalfTurnToGroup[int(j-1)] != HalfTurnToGroup[lastj-1] and lastj != 0:
+                        PL.Inputs.vQ_iStartQuench[int(j-1)] = 0
+                        PL.Inputs.lengthHotSpot_iStartQuench[int(j-1)] = 0
+                    lastj = int(j)
+
         PL.setAttribute("Inputs", "tStartQuench", tStartQuench)
 
+        if '#HeatingStations' in parameter_dict.keys():
+            [_, th_con_h, th_con_w] = PL._obtainThermalConnections()
+            PL.adjust_vQ_QuenchHeater(th_con_h, th_con_w, parameter_dict['#HeatingStations'])
+
         if 'U_inductive_dynamic_CoilSections' not in PL.Variables.variableToSaveTxt:
             PL.Variables.variableToSaveTxt = np.append(PL.Variables.variableToSaveTxt, 'U_inductive_dynamic_CoilSections')
             PL.Variables.typeVariableToSaveTxt = np.append(PL.Variables.typeVariableToSaveTxt, 1)
